"""
アプリケーション設定管理モジュール

環境変数から設定を読み込み、アプリケーション全体で使用する設定を提供します。
"""

import os
import warnings
from pathlib import Path
from typing import Literal

from dotenv import load_dotenv
from pydantic import Field, field_validator, model_validator
from pydantic_settings import BaseSettings
from typing import Any

from .exceptions import ConfigurationError

# プロジェクトルートディレクトリ
PROJECT_ROOT = Path(__file__).parent.parent.parent

# .envファイルを読み込み
try:
    load_dotenv(PROJECT_ROOT / ".env")
except Exception as e:
    # .envファイルが無くても動作するが警告を出す
    warnings.warn(
        f".env file could not be loaded: {e}. "
        "Using environment variables or defaults.",
        UserWarning
    )


class Neo4jSettings(BaseSettings):
    """Neo4jデータベース接続設定"""

    uri: str = Field(default="bolt://localhost:7687", description="Neo4j接続URI")
    user: str = Field(default="neo4j", description="Neo4jユーザー名")
    password: str = Field(..., description="Neo4jパスワード（必須）")
    database: str = Field(default="neo4j", description="Neo4jデータベース名")
    auto_create_constraints: bool = Field(
        default=True, description="制約を自動作成するか"
    )
    auto_create_indexes: bool = Field(
        default=True, description="インデックスを自動作成するか"
    )

    @field_validator('uri')
    @classmethod
    def validate_uri(cls, v: str) -> str:
        """URIスキームの検証"""
        if not v.startswith(('bolt://', 'neo4j://', 'neo4j+s://', 'neo4j+ssc://')):
            raise ValueError(
                f"Invalid Neo4j URI scheme: {v}. "
                "Must start with bolt://, neo4j://, neo4j+s://, or neo4j+ssc://"
            )
        return v

    @field_validator('password')
    @classmethod
    def validate_password(cls, v: str) -> str:
        """パスワード強度の検証"""
        if len(v) < 8:
            raise ValueError("Password must be at least 8 characters long")
        return v

    class Config:
        env_prefix = "NEO4J_"


class OllamaSettings(BaseSettings):
    """Ollama設定"""

    host: str = Field(default="http://localhost:11434", description="OllamaホストURL")
    model: str = Field(default="gpt-oss:20b", description="使用するモデル名")
    timeout: int = Field(default=120, description="タイムアウト秒数")
    temperature: float = Field(default=0.7, description="生成時の温度パラメータ")

    @field_validator('host')
    @classmethod
    def validate_host(cls, v: str) -> str:
        """ホストURLの検証"""
        if not v.startswith(('http://', 'https://')):
            raise ValueError(f"Invalid Ollama host URL: {v}. Must start with http:// or https://")
        return v

    @field_validator('timeout')
    @classmethod
    def validate_timeout(cls, v: int) -> int:
        """タイムアウト値の検証"""
        if v < 10 or v > 600:
            raise ValueError("Timeout must be between 10 and 600 seconds")
        return v

    class Config:
        env_prefix = "OLLAMA_"


class LogSettings(BaseSettings):
    """ログ設定"""

    level: str = Field(default="INFO", description="ログレベル")
    file: str = Field(default="logs/inheritance.log", description="ログファイルパス")
    max_bytes: int = Field(default=10485760, description="ログファイル最大サイズ")
    backup_count: int = Field(default=5, description="バックアップファイル数")

    @field_validator('level')
    @classmethod
    def validate_level(cls, v: str) -> str:
        """ログレベルの検証"""
        valid_levels = ["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"]
        v_upper = v.upper()
        if v_upper not in valid_levels:
            raise ValueError(f"Invalid log level: {v}. Must be one of {valid_levels}")
        return v_upper

    class Config:
        env_prefix = "LOG_"


class AppSettings(BaseSettings):
    """アプリケーション設定"""

    env: Literal["development", "production", "test"] = Field(
        default="development", description="実行環境"
    )
    debug: bool = Field(default=False, description="デバッグモード")
    name: str = Field(default="inheritance-calculator", description="アプリケーション名")

    class Config:
        env_prefix = "APP_"


class AgentSettings(BaseSettings):
    """エージェント設定"""

    max_retries: int = Field(default=3, description="最大リトライ回数")
    timeout: int = Field(default=60, description="タイムアウト秒数")

    class Config:
        env_prefix = "AGENT_"


class OutputSettings(BaseSettings):
    """出力設定"""

    dir: str = Field(default="output", description="出力ディレクトリ")
    format: Literal["json", "yaml", "csv"] = Field(
        default="json", description="デフォルト出力形式"
    )
    enable_rich: bool = Field(default=True, description="Rich出力を有効にするか")

    class Config:
        env_prefix = "OUTPUT_"


class Settings(BaseSettings):
    """統合設定クラス"""

    neo4j: Neo4jSettings = Field(default=None)  # type: ignore
    ollama: OllamaSettings = Field(default=None)  # type: ignore
    log: LogSettings = Field(default=None)  # type: ignore
    app: AppSettings = Field(default=None)  # type: ignore
    agent: AgentSettings = Field(default=None)  # type: ignore
    output: OutputSettings = Field(default=None)  # type: ignore

    @model_validator(mode='before')
    @classmethod
    def create_nested_settings(cls, data: Any) -> Any:
        """ネストされた設定を自動生成"""
        if isinstance(data, dict):
            if 'neo4j' not in data or data['neo4j'] is None:
                data['neo4j'] = Neo4jSettings()  # type: ignore
            if 'ollama' not in data or data['ollama'] is None:
                data['ollama'] = OllamaSettings()  # type: ignore
            if 'log' not in data or data['log'] is None:
                data['log'] = LogSettings()  # type: ignore
            if 'app' not in data or data['app'] is None:
                data['app'] = AppSettings()  # type: ignore
            if 'agent' not in data or data['agent'] is None:
                data['agent'] = AgentSettings()  # type: ignore
            if 'output' not in data or data['output'] is None:
                data['output'] = OutputSettings()  # type: ignore
        return data

    @property
    def project_root(self) -> Path:
        """プロジェクトルートディレクトリを取得"""
        return PROJECT_ROOT

    @property
    def logs_dir(self) -> Path:
        """ログディレクトリを取得"""
        logs_dir = PROJECT_ROOT / "logs"
        try:
            logs_dir.mkdir(exist_ok=True)
        except OSError as e:
            raise ConfigurationError(
                f"Failed to create logs directory: {logs_dir}"
            ) from e
        return logs_dir

    @property
    def output_dir(self) -> Path:
        """出力ディレクトリを取得"""
        output_dir = PROJECT_ROOT / self.output.dir
        try:
            output_dir.mkdir(exist_ok=True)
        except OSError as e:
            raise ConfigurationError(
                f"Failed to create output directory: {output_dir}"
            ) from e
        return output_dir


# シングルトンインスタンス
# 注: 実行時にはNEO4J_PASSWORD環境変数が必須
try:
    settings = Settings()
except Exception:
    # テスト環境やインポート時のエラーを回避
    # 実際の使用時には環境変数が設定されている必要がある
    settings = None  # type: ignore



"""Cypherクエリ集

Neo4jデータベースで使用するCypherクエリを定義する。
"""
from typing import Dict, Any, Optional


class PersonQueries:
    """人物(Person)ノードに関するクエリ"""

    CREATE = """
    CREATE (p:Person {
        name: $name,
        is_alive: $is_alive,
        is_decedent: $is_decedent,
        birth_date: date($birth_date),
        death_date: CASE WHEN $death_date IS NOT NULL THEN date($death_date) ELSE NULL END,
        gender: $gender,
        address: $address,
        phone: $phone,
        email: $email
    })
    RETURN p
    """

    FIND_BY_NAME = """
    MATCH (p:Person {name: $name})
    RETURN p
    """

    FIND_DECEDENT = """
    MATCH (p:Person {is_decedent: true})
    RETURN p
    """

    FIND_ALL = """
    MATCH (p:Person)
    RETURN p
    ORDER BY p.name
    """

    UPDATE = """
    MATCH (p:Person {name: $name})
    SET p.is_alive = $is_alive,
        p.death_date = CASE WHEN $death_date IS NOT NULL THEN date($death_date) ELSE NULL END,
        p.gender = $gender,
        p.address = $address,
        p.phone = $phone,
        p.email = $email
    RETURN p
    """

    DELETE = """
    MATCH (p:Person {name: $name})
    DETACH DELETE p
    """

    DELETE_ALL = """
    MATCH (p:Person)
    DETACH DELETE p
    """


class RelationshipQueries:
    """リレーションシップに関するクエリ"""

    # 親子関係
    CREATE_CHILD_OF = """
    MATCH (child:Person {name: $child_name})
    MATCH (parent:Person {name: $parent_name})
    CREATE (child)-[r:CHILD_OF {
        adoption: $adoption,
        is_biological: $is_biological
    }]->(parent)
    RETURN r
    """

    FIND_CHILDREN = """
    MATCH (parent:Person {name: $parent_name})<-[:CHILD_OF]-(child:Person)
    RETURN child
    ORDER BY child.name
    """

    FIND_PARENTS = """
    MATCH (child:Person {name: $child_name})-[:CHILD_OF]->(parent:Person)
    RETURN parent
    ORDER BY parent.name
    """

    # 配偶者関係
    CREATE_SPOUSE_OF = """
    MATCH (person1:Person {name: $person1_name})
    MATCH (person2:Person {name: $person2_name})
    CREATE (person1)-[r:SPOUSE_OF {
        marriage_date: CASE WHEN $marriage_date IS NOT NULL THEN date($marriage_date) ELSE NULL END,
        divorce_date: CASE WHEN $divorce_date IS NOT NULL THEN date($divorce_date) ELSE NULL END,
        is_current: $is_current
    }]->(person2)
    RETURN r
    """

    FIND_SPOUSE = """
    MATCH (person:Person {name: $person_name})-[r:SPOUSE_OF]-(spouse:Person)
    WHERE r.is_current = true
    RETURN spouse
    """

    # 兄弟姉妹関係
    CREATE_SIBLING_OF = """
    MATCH (person1:Person {name: $person1_name})
    MATCH (person2:Person {name: $person2_name})
    CREATE (person1)-[r:SIBLING_OF {
        blood_type: $blood_type,
        shared_parent: $shared_parent
    }]->(person2)
    RETURN r
    """

    FIND_SIBLINGS = """
    MATCH (person:Person {name: $person_name})-[:SIBLING_OF]-(sibling:Person)
    RETURN sibling, r
    ORDER BY sibling.name
    """

    # 相続放棄
    CREATE_RENOUNCED = """
    MATCH (person:Person {name: $person_name})
    MATCH (decedent:Person {name: $decedent_name})
    CREATE (person)-[r:RENOUNCED {
        renounce_date: date($renounce_date),
        reason: $reason
    }]->(decedent)
    RETURN r
    """

    # 相続欠格
    CREATE_DISQUALIFIED = """
    MATCH (person:Person {name: $person_name})
    MATCH (decedent:Person {name: $decedent_name})
    CREATE (person)-[r:DISQUALIFIED {
        reason: $reason,
        date: date($date)
    }]->(decedent)
    RETURN r
    """

    # 相続廃除
    CREATE_DISINHERITED = """
    MATCH (person:Person {name: $person_name})
    MATCH (decedent:Person {name: $decedent_name})
    CREATE (person)-[r:DISINHERITED {
        reason: $reason,
        court_decision_date: date($court_decision_date)
    }]->(decedent)
    RETURN r
    """


class InheritanceQueries:
    """相続計算に関するクエリ"""

    # 配偶者取得
    GET_SPOUSE = """
    MATCH (decedent:Person {is_decedent: true})-[r:SPOUSE_OF]-(spouse:Person)
    WHERE spouse.is_alive = true
      AND r.is_current = true
      AND NOT EXISTS((spouse)-[:RENOUNCED]->(decedent))
    RETURN spouse
    """

    # 第1順位相続人（子）取得
    GET_FIRST_RANK_HEIRS = """
    MATCH (decedent:Person {is_decedent: true})<-[:CHILD_OF]-(child:Person)
    WHERE child.is_alive = true
      AND NOT EXISTS((child)-[:RENOUNCED]->(decedent))
      AND NOT EXISTS((child)-[:DISQUALIFIED]->(decedent))
      AND NOT EXISTS((child)-[:DISINHERITED]->(decedent))
    RETURN child
    ORDER BY child.name
    """

    # 代襲相続人取得（子の代襲）
    GET_SUBSTITUTION_HEIRS_CHILDREN = """
    MATCH (decedent:Person {is_decedent: true})<-[:CHILD_OF]-(child:Person)
    WHERE child.is_alive = false
      AND child.death_date < decedent.death_date
      AND NOT EXISTS((child)-[:RENOUNCED]->(decedent))
    MATCH (child)<-[:CHILD_OF*]-(descendant:Person)
    WHERE descendant.is_alive = true
      AND NOT EXISTS((descendant)-[:RENOUNCED]->(decedent))
      AND NOT EXISTS((descendant)-[:DISQUALIFIED]->(decedent))
      AND NOT EXISTS((descendant)-[:DISINHERITED]->(decedent))
    RETURN descendant, child, size((child)<-[:CHILD_OF*]-(descendant)) as generation
    ORDER BY generation ASC, descendant.name
    """

    # 第2順位相続人（直系尊属）取得
    GET_SECOND_RANK_HEIRS = """
    MATCH (decedent:Person {is_decedent: true})
    WHERE NOT EXISTS((decedent)<-[:CHILD_OF]-(:Person {is_alive: true}))
      AND NOT EXISTS((decedent)<-[:CHILD_OF]-(:Person {is_alive: false})<-[:CHILD_OF]-(:Person {is_alive: true}))
    MATCH (decedent)-[:CHILD_OF*]->(ancestor:Person)
    WHERE ancestor.is_alive = true
      AND NOT EXISTS((ancestor)-[:RENOUNCED]->(decedent))
    WITH ancestor, size((decedent)-[:CHILD_OF*]->(ancestor)) as generation
    WITH min(generation) as min_gen
    MATCH (decedent)-[:CHILD_OF*]->(ancestor:Person)
    WHERE ancestor.is_alive = true
      AND size((decedent)-[:CHILD_OF*]->(ancestor)) = min_gen
      AND NOT EXISTS((ancestor)-[:RENOUNCED]->(decedent))
    RETURN ancestor
    ORDER BY ancestor.name
    """

    # 第3順位相続人（兄弟姉妹）取得
    GET_THIRD_RANK_HEIRS = """
    MATCH (decedent:Person {is_decedent: true})
    WHERE NOT EXISTS((decedent)<-[:CHILD_OF]-(:Person {is_alive: true}))
      AND NOT EXISTS((decedent)<-[:CHILD_OF]-(:Person {is_alive: false})<-[:CHILD_OF]-(:Person {is_alive: true}))
      AND NOT EXISTS((decedent)-[:CHILD_OF*]->(:Person {is_alive: true}))
    MATCH (decedent)-[r:SIBLING_OF]-(sibling:Person)
    WHERE sibling.is_alive = true
      AND NOT EXISTS((sibling)-[:RENOUNCED]->(decedent))
      AND NOT EXISTS((sibling)-[:DISQUALIFIED]->(decedent))
      AND NOT EXISTS((sibling)-[:DISINHERITED]->(decedent))
    RETURN sibling, r.blood_type as blood_type
    ORDER BY sibling.name
    """

    # 代襲相続人取得（兄弟姉妹の代襲：1代限り）
    GET_SUBSTITUTION_HEIRS_SIBLINGS = """
    MATCH (decedent:Person {is_decedent: true})
    WHERE NOT EXISTS((decedent)<-[:CHILD_OF]-(:Person {is_alive: true}))
      AND NOT EXISTS((decedent)<-[:CHILD_OF]-(:Person {is_alive: false})<-[:CHILD_OF]-(:Person {is_alive: true}))
      AND NOT EXISTS((decedent)-[:CHILD_OF*]->(:Person {is_alive: true}))
    MATCH (decedent)-[r:SIBLING_OF]-(sibling:Person)
    WHERE sibling.is_alive = false
      AND sibling.death_date < decedent.death_date
      AND NOT EXISTS((sibling)-[:RENOUNCED]->(decedent))
    MATCH (sibling)<-[:CHILD_OF]-(nephew_niece:Person)
    WHERE nephew_niece.is_alive = true
      AND NOT EXISTS((nephew_niece)-[:RENOUNCED]->(decedent))
      AND NOT EXISTS((nephew_niece)-[:DISQUALIFIED]->(decedent))
      AND NOT EXISTS((nephew_niece)-[:DISINHERITED]->(decedent))
    RETURN nephew_niece, sibling, r.blood_type as blood_type
    ORDER BY nephew_niece.name
    """

    # 家系図取得（全リレーションシップ）
    GET_FAMILY_TREE = """
    MATCH (decedent:Person {is_decedent: true})
    OPTIONAL MATCH (decedent)-[r1]-(related:Person)
    OPTIONAL MATCH (related)-[r2]-(indirect:Person)
    WHERE indirect <> decedent
    RETURN decedent, r1, related, r2, indirect
    """

    # 統計情報取得
    GET_STATISTICS = """
    MATCH (p:Person)
    WITH count(p) as total_persons,
         count(CASE WHEN p.is_alive THEN 1 END) as alive_persons,
         count(CASE WHEN p.is_decedent THEN 1 END) as decedent_count
    MATCH ()-[r]->()
    WITH total_persons, alive_persons, decedent_count,
         count(r) as total_relationships
    RETURN total_persons, alive_persons, decedent_count, total_relationships
    """


class GraphQueries:
    """グラフ全体に関するクエリ"""

    # 全ノード・リレーションシップ削除（テスト用）
    DELETE_ALL = """
    MATCH (n)
    DETACH DELETE n
    """

    # 制約とインデックスの作成
    CREATE_CONSTRAINTS = [
        "CREATE CONSTRAINT person_name_unique IF NOT EXISTS FOR (p:Person) REQUIRE p.name IS UNIQUE",
        "CREATE INDEX person_name_index IF NOT EXISTS FOR (p:Person) ON (p.name)",
        "CREATE INDEX person_decedent_index IF NOT EXISTS FOR (p:Person) ON (p.is_decedent)",
        "CREATE INDEX person_alive_index IF NOT EXISTS FOR (p:Person) ON (p.is_alive)",
    ]

    # データベース情報取得
    GET_DATABASE_INFO = """
    CALL dbms.components() YIELD name, versions, edition
    RETURN name, versions, edition
    """


def build_person_params(
    name: str,
    is_alive: bool = True,
    is_decedent: bool = False,
    birth_date: Optional[str] = None,
    death_date: Optional[str] = None,
    gender: Optional[str] = None,
    address: Optional[str] = None,
    phone: Optional[str] = None,
    email: Optional[str] = None
) -> Dict[str, Any]:
    """
    Person作成用のパラメータを構築

    Args:
        name: 氏名
        is_alive: 生存状態
        is_decedent: 被相続人フラグ
        birth_date: 生年月日（YYYY-MM-DD形式）
        death_date: 死亡日（YYYY-MM-DD形式）
        gender: 性別
        address: 住所
        phone: 電話番号
        email: メールアドレス

    Returns:
        Cypherクエリ用のパラメータ辞書
    """
    return {
        "name": name,
        "is_alive": is_alive,
        "is_decedent": is_decedent,
        "birth_date": birth_date,
        "death_date": death_date,
        "gender": gender,
        "address": address,
        "phone": phone,
        "email": email
    }



"""相続計算結果モデル"""
from typing import List, Optional, Any
from fractions import Fraction
from enum import Enum
from uuid import UUID

from pydantic import BaseModel, Field, field_validator, ConfigDict

from .person import Person


class HeritageRank(str, Enum):
    """相続順位"""
    SPOUSE = "spouse"  # 配偶者（常に相続人）
    FIRST = "first"  # 第1順位（子・直系卑属）
    SECOND = "second"  # 第2順位（直系尊属）
    THIRD = "third"  # 第3順位（兄弟姉妹）


class SubstitutionType(str, Enum):
    """代襲相続タイプ"""
    NONE = "none"  # 代襲なし
    CHILD = "child"  # 子の代襲（孫、曾孫...）
    SIBLING = "sibling"  # 兄弟姉妹の代襲（甥・姪のみ）


class Heir(BaseModel):
    """
    相続人

    相続人の資格と相続割合を表現。
    """

    model_config = ConfigDict(
        validate_assignment=True,
        extra='forbid',
    )

    person: Person = Field(..., description="相続人")
    rank: HeritageRank = Field(..., description="相続順位")
    share: Fraction = Field(..., description="相続割合（法定相続分）")
    share_percentage: float = Field(..., description="相続割合（百分率）")
    is_substitution: bool = Field(default=False, description="代襲相続人か")
    substitution_type: SubstitutionType = Field(
        default=SubstitutionType.NONE, description="代襲相続タイプ"
    )
    substituted_person: Optional[Person] = Field(
        default=None, description="被代襲者（代襲相続の場合）"
    )
    generation: int = Field(default=1, description="代襲世代数（1=直接相続人）")
    is_retransfer: bool = Field(default=False, description="再転相続人か")
    retransfer_from: Optional[Person] = Field(
        default=None, description="再転相続元の相続人（遺産分割前に死亡した相続人）"
    )
    original_share: Optional[Fraction] = Field(
        default=None, description="再転相続前の元の相続分"
    )

    @field_validator('share_percentage', mode='before')
    @classmethod
    def calculate_percentage(cls, v: Any, info: Any) -> float:
        """相続割合の百分率を計算"""
        # shareから計算（vが0.0の場合も含む）
        values = info.data
        share = values.get('share')
        if share is not None:
            return float(share) * 100

        # すでに計算されている場合はそのまま返す
        if isinstance(v, float):
            return v

        return 0.0

    @field_validator('substitution_type')
    @classmethod
    def validate_substitution_type(cls, v: SubstitutionType, info: Any) -> SubstitutionType:
        """代襲相続タイプの検証"""
        values = info.data
        is_substitution = values.get('is_substitution', False)

        # 代襲相続でない場合、タイプはNONEでなければならない
        if not is_substitution and v != SubstitutionType.NONE:
            raise ValueError("Non-substitution heir must have NONE type")

        # 代襲相続の場合、タイプはNONE以外でなければならない
        if is_substitution and v == SubstitutionType.NONE:
            raise ValueError("Substitution heir must have valid substitution type")

        return v

    def __str__(self) -> str:
        """文字列表現"""
        substitution_mark = ""
        if self.is_substitution:
            substitution_mark = f"（代襲: {self.substituted_person.name if self.substituted_person else '不明'}）"

        share_str = f"{self.share} ({self.share_percentage:.2f}%)"
        return f"{self.person.name} - {self.rank.value} - {share_str}{substitution_mark}"


class InheritanceResult(BaseModel):
    """
    相続計算結果

    相続人の確定と相続割合の計算結果を表現。
    """

    model_config = ConfigDict(
        validate_assignment=True,
        extra='forbid',
    )

    decedent: Person = Field(..., description="被相続人")
    heirs: List[Heir] = Field(default_factory=list, description="相続人リスト")
    has_spouse: bool = Field(default=False, description="配偶者がいるか")
    has_children: bool = Field(default=False, description="子がいるか")
    has_parents: bool = Field(default=False, description="直系尊属がいるか")
    has_siblings: bool = Field(default=False, description="兄弟姉妹がいるか")
    calculation_basis: List[str] = Field(
        default_factory=list, description="計算根拠（適用した民法条文等）"
    )

    @field_validator('heirs')
    @classmethod
    def validate_total_share(cls, v: List[Heir]) -> List[Heir]:
        """相続割合の合計が1（100%）であることを検証"""
        if not v:
            return v

        total = sum(heir.share for heir in v)
        if total != Fraction(1, 1):
            raise ValueError(
                f"Total share must be 1 (100%), but got {total} ({float(total) * 100}%)"
            )

        return v

    def add_heir(
        self,
        person: Person,
        rank: HeritageRank,
        share: Fraction,
        is_substitution: bool = False,
        substitution_type: SubstitutionType = SubstitutionType.NONE,
        substituted_person: Optional[Person] = None,
        generation: int = 1
    ) -> None:
        """
        相続人を追加

        Args:
            person: 相続人
            rank: 相続順位
            share: 相続割合
            is_substitution: 代襲相続人か
            substitution_type: 代襲相続タイプ
            substituted_person: 被代襲者
            generation: 代襲世代数
        """
        heir = Heir(
            person=person,
            rank=rank,
            share=share,
            share_percentage=float(share) * 100,
            is_substitution=is_substitution,
            substitution_type=substitution_type,
            substituted_person=substituted_person,
            generation=generation
        )
        self.heirs.append(heir)

    def add_calculation_basis(self, basis: str) -> None:
        """
        計算根拠を追加

        Args:
            basis: 計算根拠（民法条文等）
        """
        self.calculation_basis.append(basis)

    def get_heirs_by_rank(self, rank: HeritageRank) -> List[Heir]:
        """
        指定された順位の相続人を取得

        Args:
            rank: 相続順位

        Returns:
            相続人のリスト
        """
        return [heir for heir in self.heirs if heir.rank == rank]

    def get_substitution_heirs(self) -> List[Heir]:
        """
        代襲相続人のリストを取得

        Returns:
            代襲相続人のリスト
        """
        return [heir for heir in self.heirs if heir.is_substitution]

    @property
    def total_heirs(self) -> int:
        """相続人の総数"""
        return len(self.heirs)

    def __str__(self) -> str:
        """文字列表現"""
        result = [f"被相続人: {self.decedent.name}"]
        result.append(f"相続人総数: {self.total_heirs}名")
        result.append("---")

        for heir in self.heirs:
            result.append(str(heir))

        if self.calculation_basis:
            result.append("---")
            result.append("計算根拠:")
            for basis in self.calculation_basis:
                result.append(f"  - {basis}")

        return "\n".join(result)



"""Models package exports"""
from .person import Person, Gender
from .relationship import BloodType
from .inheritance import InheritanceResult, Heir
from .value_objects import PersonID

__all__ = [
    "Person",
    "Gender",
    "BloodType",
    "InheritanceResult",
    "Heir",
    "PersonID",
]



"""相続計算サービス

相続人の確定と相続割合の計算を統合して実行するサービス。
"""
from typing import List, Dict, Optional
from fractions import Fraction

from ..models.person import Person
from ..models.relationship import BloodType
from ..models.inheritance import (
    InheritanceResult,
    HeritageRank,
    SubstitutionType,
    Heir,
)
from ..models.value_objects import PersonID
from ..utils.exceptions import RenunciationConflictError
from .heir_validator import HeirValidator
from .share_calculator import ShareCalculator
from .base import BaseService


class InheritanceCalculator(BaseService[InheritanceResult]):
    """
    相続計算サービス

    相続人の資格確定、相続割合の計算、結果の生成を統合的に実行する。
    """

    def __init__(
        self,
        validator: Optional[HeirValidator] = None,
        calculator: Optional[ShareCalculator] = None
    ) -> None:
        """
        初期化

        Args:
            validator: 相続人バリデータ（省略時は新規インスタンスを作成）
            calculator: 相続割合計算サービス（省略時は新規インスタンスを作成）
        """
        super().__init__()
        self.validator = validator if validator is not None else HeirValidator()
        self.calculator = calculator if calculator is not None else ShareCalculator()

    def calculate(
        self,
        decedent: Person,
        spouses: List[Person],
        children: List[Person],
        parents: List[Person],
        siblings: List[Person],
        renounced: Optional[List[Person]] = None,
        disqualified: Optional[List[Person]] = None,
        disinherited: Optional[List[Person]] = None,
        sibling_blood_types: Optional[Dict[PersonID, BloodType]] = None,
        retransfer_heirs_info: Optional[Dict[PersonID, List[Person]]] = None,
        retransfer_relationships: Optional[Dict[PersonID, Dict[PersonID, str]]] = None,
        second_inheritance_renounced: Optional[Dict[PersonID, List[Person]]] = None,
    ) -> InheritanceResult:
        """
        相続計算を実行

        Args:
            decedent: 被相続人
            spouses: 配偶者候補
            children: 子候補
            parents: 直系尊属候補
            siblings: 兄弟姉妹候補
            renounced: 相続放棄者
            disqualified: 相続欠格者
            disinherited: 相続廃除者
            sibling_blood_types: 兄弟姉妹の血縁タイプ（PersonID → BloodType）
            retransfer_heirs_info: 再転相続先の情報（相続人PersonID → 再転相続先リスト）
            retransfer_relationships: 再転相続先の関係情報（相続人PersonID → {人物PersonID → 関係タイプ}）
            second_inheritance_renounced: 第2次相続の放棄者情報（死亡相続人PersonID → 放棄者リスト）
                判例により、第2次相続を放棄した者は第1次相続のみを承認できない

        Returns:
            相続計算結果
        """
        # デフォルト値の設定
        if renounced is None:
            renounced = []
        if disqualified is None:
            disqualified = []
        if disinherited is None:
            disinherited = []
        if sibling_blood_types is None:
            sibling_blood_types = {}
        if retransfer_heirs_info is None:
            retransfer_heirs_info = {}
        if retransfer_relationships is None:
            retransfer_relationships = {}
        if second_inheritance_renounced is None:
            second_inheritance_renounced = {}

        # バリデータの初期化
        self.validator.set_decedent(decedent)
        self.validator.renounced_persons = renounced
        self.validator.disqualified_persons = disqualified
        self.validator.disinherited_persons = disinherited

        # 結果オブジェクトの作成
        result = InheritanceResult(decedent=decedent)

        # 各順位の相続人を確定
        valid_spouses = self._validate_spouses(spouses, result)
        valid_children = self._validate_children(children, result)
        valid_parents = self._validate_parents(parents, result, bool(valid_children))
        valid_siblings = self._validate_siblings(
            siblings, result, bool(valid_children), bool(valid_parents)
        )

        # 相続割合を計算
        shares = self.calculator.calculate_shares(
            valid_spouses,
            valid_children,
            valid_parents,
            valid_siblings,
            sibling_blood_types
        )

        # 相続人を結果に追加
        self._add_heirs_to_result(
            result,
            valid_spouses,
            valid_children,
            valid_parents,
            valid_siblings,
            shares
        )

        # フラグを設定
        result.has_spouse = len(valid_spouses) > 0
        result.has_children = len(valid_children) > 0
        result.has_parents = len(valid_parents) > 0
        result.has_siblings = len(valid_siblings) > 0

        # 再転相続の処理
        if retransfer_heirs_info:
            all_persons = spouses + children + parents + siblings
            result = self._process_retransfer_inheritance_with_info(
                result, retransfer_heirs_info, retransfer_relationships, second_inheritance_renounced
            )

        self.log_operation(
            "Inheritance calculation completed",
            total_heirs=result.total_heirs
        )

        return result

    def _validate_spouses(
        self, spouses: List[Person], result: InheritanceResult
    ) -> List[Person]:
        """配偶者の資格検証"""
        valid_spouses = []
        for spouse in spouses:
            if self.validator.validate_spouse(spouse):
                valid_spouses.append(spouse)
                result.add_calculation_basis("民法890条（配偶者の相続権）")

        return valid_spouses

    def _validate_children(
        self, children: List[Person], result: InheritanceResult
    ) -> List[Person]:
        """子の資格検証"""
        valid_children = []
        for child in children:
            if self.validator.validate_child(child):
                valid_children.append(child)

        if valid_children:
            result.add_calculation_basis("民法887条1項（子の相続権）")

        return valid_children

    def _validate_parents(
        self,
        parents: List[Person],
        result: InheritanceResult,
        has_first_rank: bool
    ) -> List[Person]:
        """直系尊属の資格検証"""
        valid_parents = []
        for parent in parents:
            if self.validator.validate_parent(parent, has_first_rank):
                valid_parents.append(parent)

        if valid_parents:
            result.add_calculation_basis("民法889条1項1号（直系尊属の相続権）")

        return valid_parents

    def _validate_siblings(
        self,
        siblings: List[Person],
        result: InheritanceResult,
        has_first_rank: bool,
        has_second_rank: bool
    ) -> List[Person]:
        """兄弟姉妹の資格検証"""
        valid_siblings = []
        for sibling in siblings:
            if self.validator.validate_sibling(
                sibling, has_first_rank, has_second_rank
            ):
                valid_siblings.append(sibling)

        if valid_siblings:
            result.add_calculation_basis("民法889条1項2号（兄弟姉妹の相続権）")

        return valid_siblings

    def _add_heirs_to_result(
        self,
        result: InheritanceResult,
        spouses: List[Person],
        children: List[Person],
        parents: List[Person],
        siblings: List[Person],
        shares: Dict[PersonID, Fraction]
    ) -> None:
        """相続人を結果に追加"""
        # 配偶者
        for spouse in spouses:
            share = shares.get(spouse.id, Fraction(0, 1))
            result.add_heir(spouse, HeritageRank.SPOUSE, share)

        # 子
        for child in children:
            share = shares.get(child.id, Fraction(0, 1))
            result.add_heir(child, HeritageRank.FIRST, share)

        # 直系尊属
        for parent in parents:
            share = shares.get(parent.id, Fraction(0, 1))
            result.add_heir(parent, HeritageRank.SECOND, share)

        # 兄弟姉妹
        for sibling in siblings:
            share = shares.get(sibling.id, Fraction(0, 1))
            result.add_heir(sibling, HeritageRank.THIRD, share)

        # 相続分の計算根拠を追加
        if spouses and children:
            result.add_calculation_basis("民法900条1号（配偶者1/2、子1/2）")
        elif spouses and parents:
            result.add_calculation_basis("民法900条2号（配偶者2/3、直系尊属1/3）")
        elif spouses and siblings:
            result.add_calculation_basis("民法900条3号（配偶者3/4、兄弟姉妹1/4）")

    def _process_retransfer_inheritance_with_info(
        self,
        result: InheritanceResult,
        retransfer_info: Dict[PersonID, List[Person]],
        retransfer_relationships: Dict[PersonID, Dict[PersonID, str]],
        second_inheritance_renounced: Dict[PersonID, List[Person]]
    ) -> InheritanceResult:
        """
        再転相続の処理（情報付き版）

        Args:
            result: 現在の相続計算結果
            retransfer_info: 再転相続先の情報（相続人ID: 再転相続先リスト）
            retransfer_relationships: 再転相続先の関係情報（相続人ID: {人物ID: 関係タイプ}）
            second_inheritance_renounced: 第2次相続の放棄者情報（死亡相続人ID: 放棄者リスト）

        Returns:
            再転相続処理後の相続計算結果
        """
        # 遺産分割前に死亡した相続人を特定
        retransfer_heirs = [
            heir for heir in result.heirs
            if heir.person.died_before_division and not heir.person.is_alive
        ]

        if not retransfer_heirs:
            return result

        # 再転相続が発生した旨を記録
        result.add_calculation_basis("民法第896条（相続人の相続、再転相続）")

        new_heirs = []

        # 再転相続が発生しない相続人をそのまま追加
        for heir in result.heirs:
            if not heir.person.died_before_division:
                new_heirs.append(heir)

        # 再転相続の処理
        for original_heir in retransfer_heirs:
            heir_id = original_heir.person.id
            retransfer_targets = retransfer_info.get(heir_id, [])

            if not retransfer_targets:
                # 再転相続先がいない場合は元の相続人をそのまま
                new_heirs.append(original_heir)
                continue

            # 判例制約の検証: 第2次相続を放棄した者が第1次相続の再転相続先に含まれていないか
            self._validate_retransfer_renunciation(
                result.decedent,
                original_heir.person,
                retransfer_targets,
                second_inheritance_renounced.get(heir_id, [])
            )

            # 再転相続先を相続順位別に分類
            # retransfer_relationships から該当する相続人の関係情報を取得
            relationship_hints = retransfer_relationships.get(heir_id, None)

            classified_heirs = self._classify_retransfer_heirs(
                retransfer_targets,
                original_heir.person,
                relationship_hints
            )

            # 再転相続分を計算（法定相続分に基づく）
            retransfer_shares = self._calculate_retransfer_shares_classified(
                original_heir.share,
                classified_heirs
            )

            # 再転相続人を追加
            for target, share in retransfer_shares:
                new_heir = Heir(
                    person=target,
                    rank=original_heir.rank,
                    share=share,
                    share_percentage=float(share) * 100,
                    is_retransfer=True,
                    retransfer_from=original_heir.person,
                    original_share=original_heir.share
                )
                new_heirs.append(new_heir)

        # 相続人リストを更新
        result.heirs = new_heirs

        return result

    def _process_retransfer_inheritance(
        self,
        result: InheritanceResult,
        all_persons: List[Person]
    ) -> InheritanceResult:
        """
        再転相続の処理

        遺産分割前に死亡した相続人がいる場合、
        その相続分をその相続人の相続人に再転相続させる。

        Args:
            result: 現在の相続計算結果
            all_persons: 再転相続先の候補となる全人物リスト

        Returns:
            再転相続処理後の相続計算結果
        """
        # 遺産分割前に死亡した相続人を特定
        retransfer_heirs = [
            heir for heir in result.heirs
            if heir.person.died_before_division and not heir.person.is_alive
        ]

        if not retransfer_heirs:
            return result

        # 再転相続が発生した旨を記録
        result.add_calculation_basis("民法第896条（相続人の相続、再転相続）")

        new_heirs = []

        # 再転相続が発生しない相続人をそのまま追加
        for heir in result.heirs:
            if not heir.person.died_before_division:
                new_heirs.append(heir)

        # 再転相続の処理
        for original_heir in retransfer_heirs:
            # この相続人の相続人を探す
            retransfer_targets = self._find_retransfer_heirs(
                original_heir.person,
                all_persons
            )

            if not retransfer_targets:
                # 再転相続先がいない場合は元の相続人をそのまま
                new_heirs.append(original_heir)
                continue

            # 再転相続分を計算
            retransfer_shares = self._calculate_retransfer_shares(
                original_heir.share,
                retransfer_targets
            )

            # 再転相続人を追加
            for target, share in retransfer_shares:
                new_heir = Heir(
                    person=target,
                    rank=original_heir.rank,
                    share=share,
                    share_percentage=float(share) * 100,
                    is_retransfer=True,
                    retransfer_from=original_heir.person,
                    original_share=original_heir.share
                )
                new_heirs.append(new_heir)

        # 相続人リストを更新
        result.heirs = new_heirs

        return result

    def _find_retransfer_heirs(
        self,
        deceased_heir: Person,
        all_persons: List[Person]
    ) -> List[Person]:
        """
        再転相続先の相続人を探す

        Args:
            deceased_heir: 遺産分割前に死亡した相続人
            all_persons: 全人物リスト

        Returns:
            再転相続先の相続人リスト
        """
        # 簡易実装: 配偶者と子を再転相続先とする
        # 実際にはHeirValidatorを使って正確に判定すべき
        retransfer_heirs = []

        for person in all_persons:
            if person.is_alive and person.id != deceased_heir.id:
                # この実装では、配偶者・子・親などの関係を
                # 外部から渡される必要がある
                # 簡易実装として、すべての生存者を候補とする
                retransfer_heirs.append(person)

        return retransfer_heirs

    def _validate_retransfer_renunciation(
        self,
        decedent: Person,
        deceased_heir: Person,
        retransfer_targets: List[Person],
        second_inheritance_renounced: List[Person]
    ) -> None:
        """
        再転相続における相続放棄の制約を検証

        判例（最高裁昭和63年6月21日判決）により、再転相続において
        第2次相続（相続人の相続）を放棄した者は、第1次相続（被相続人の相続）
        のみを承認することはできない。

        Args:
            decedent: 被相続人（第1次相続の被相続人）
            deceased_heir: 遺産分割前に死亡した相続人（第2次相続の被相続人）
            retransfer_targets: 再転相続先（第1次相続を承認しようとしている者）
            second_inheritance_renounced: 第2次相続の放棄者リスト

        Raises:
            RenunciationConflictError: 第2次相続を放棄した者が第1次相続を承認しようとしている場合
        """
        for renounced_person in second_inheritance_renounced:
            if renounced_person in retransfer_targets:
                raise RenunciationConflictError(
                    f"{renounced_person.name}は{deceased_heir.name}の相続を放棄しているため、"
                    f"{decedent.name}の相続のみを承認することはできません "
                    f"（最高裁昭和63年6月21日判決）。\n"
                    f"再転相続においては、第2次相続を放棄した者は第1次相続のみを単独で承認できません。"
                )

    def _classify_retransfer_heirs(
        self,
        retransfer_targets: List[Person],
        deceased_heir: Person,
        relationship_hints: Optional[Dict[PersonID, str]] = None
    ) -> Dict[str, List[Person]]:
        """
        再転相続先を相続順位別に分類

        relationship_hintsが提供されている場合は、それを使用して分類する。
        提供されていない場合は、暫定的に全員を子として扱う（後方互換性）。

        Args:
            retransfer_targets: 再転相続先のリスト
            deceased_heir: 遺産分割前に死亡した相続人
            relationship_hints: 人物IDから関係タイプへのマッピング
                キー: PersonID
                値: 'spouse' | 'child' | 'parent' | 'sibling'

        Returns:
            相続順位別の分類 {'spouses': [...], 'children': [...], ...}
        """
        classified: Dict[str, List[Person]] = {
            'spouses': [],
            'children': [],
            'parents': [],
            'siblings': []
        }

        # relationship_hintsが提供されていない場合は全員を子として扱う（後方互換性）
        if relationship_hints is None:
            classified['children'] = retransfer_targets
            return classified

        # relationship_hintsを使って分類
        for person in retransfer_targets:
            person_id = person.id
            relationship = relationship_hints.get(person_id, 'child')  # デフォルトは子

            if relationship == 'spouse':
                classified['spouses'].append(person)
            elif relationship == 'child':
                classified['children'].append(person)
            elif relationship == 'parent':
                classified['parents'].append(person)
            elif relationship == 'sibling':
                classified['siblings'].append(person)
            else:
                # 不明な関係タイプの場合は子として扱う
                classified['children'].append(person)

        return classified

    def _calculate_retransfer_shares_classified(
        self,
        original_share: Fraction,
        classified_heirs: Dict[str, List[Person]]
    ) -> List[tuple[Person, Fraction]]:
        """
        分類済み再転相続先の相続分を計算（民法第896条に基づく）

        遺産分割前に死亡した相続人の相続分を、その相続人の法定相続人に
        法定相続分に従って分配する。

        計算フロー:
        1. 分類済みの相続人リストを使用
        2. ShareCalculatorで法定相続分を計算
        3. 元の相続分に法定相続分を乗じて最終的な相続分を算出

        Args:
            original_share: 元の相続分（遺産分割前に死亡した相続人の相続分）
            classified_heirs: 相続順位別に分類された再転相続先

        Returns:
            各再転相続人と相続分のタプルのリスト

        Example:
            元の相続分が1/1で、配偶者1人・子2人の場合:
            - 配偶者: 1/1 × 1/2 = 1/2（民法900条1号）
            - 子1: 1/1 × 1/4 = 1/4（民法900条1号）
            - 子2: 1/1 × 1/4 = 1/4（民法900条1号）
        """
        spouses = classified_heirs.get('spouses', [])
        children = classified_heirs.get('children', [])
        parents = classified_heirs.get('parents', [])
        siblings = classified_heirs.get('siblings', [])

        # 再転相続先がいない場合
        if not (spouses or children or parents or siblings):
            return []

        # ShareCalculatorを使って法定相続分を計算
        statutory_shares = self.calculator.calculate_shares(
            spouses=spouses,
            first_rank=children,
            second_rank=parents,
            third_rank=siblings,
            third_rank_blood_types={}  # TODO: 必要に応じて血縁タイプ情報を渡す
        )

        # 元の相続分を法定相続分で按分
        result = []
        all_heirs = spouses + children + parents + siblings
        for person in all_heirs:
            person_id = person.id
            if person_id in statutory_shares:
                # 元の相続分 × 再転相続先の法定相続分
                final_share = original_share * statutory_shares[person_id]
                result.append((person, final_share))

        return result

    def _calculate_retransfer_shares(
        self,
        original_share: Fraction,
        retransfer_targets: List[Person]
    ) -> List[tuple[Person, Fraction]]:
        """
        再転相続分を計算（後方互換性のためのラッパー）

        Args:
            original_share: 元の相続分
            retransfer_targets: 再転相続先のリスト

        Returns:
            各再転相続人と相続分のタプルのリスト
        """
        # 暫定的な分類（全員を子として扱う）
        classified: Dict[str, List[Person]] = {
            'spouses': [],
            'children': retransfer_targets,
            'parents': [],
            'siblings': []
        }
        return self._calculate_retransfer_shares_classified(original_share, classified)



"""カスタム例外クラス"""


class InheritanceCalculatorError(Exception):
    """基底例外クラス"""
    pass


class ConfigurationError(InheritanceCalculatorError):
    """設定関連のエラー"""
    pass


class DatabaseConnectionError(InheritanceCalculatorError):
    """データベース接続エラー"""
    pass


class ValidationError(InheritanceCalculatorError):
    """バリデーションエラー"""
    pass


class LoggingError(InheritanceCalculatorError):
    """ロギング関連のエラー"""
    pass


class ServiceException(InheritanceCalculatorError):
    """サービス層のエラー"""
    pass


class DatabaseException(InheritanceCalculatorError):
    """データベース操作のエラー

    Neo4j操作、クエリ実行、トランザクション処理などで
    発生するデータベース関連のエラー。
    """
    pass


class RenunciationConflictError(ValidationError):
    """再転相続における相続放棄の制約違反エラー

    判例（最高裁昭和63年6月21日判決）により、再転相続において
    第2次相続（相続人の相続）を放棄した者は、第1次相続（被相続人の相続）
    のみを承認することはできない。

    この制約に違反した場合に発生する。
    """
    pass



"""
ログ管理モジュール

アプリケーション全体で使用するロガーを提供します。
"""

import logging
import sys
from logging.handlers import RotatingFileHandler
from pathlib import Path
from typing import Optional

from .config import settings
from .exceptions import LoggingError


def setup_logger(
    name: str,
    log_file: Optional[Path] = None,
    level: Optional[str] = None,
) -> logging.Logger:
    """
    ロガーをセットアップします。

    Args:
        name: ロガー名
        log_file: ログファイルパス（Noneの場合は設定ファイルから取得）
        level: ログレベル（Noneの場合は設定ファイルから取得）

    Returns:
        設定済みのロガー

    Raises:
        LoggingError: ロガーのセットアップに失敗した場合
    """
    try:
        logger = logging.getLogger(name)

        # 既にハンドラが設定されている場合はクリア（再設定可能に）
        if logger.handlers:
            logger.handlers.clear()

        # ログレベルの設定
        try:
            log_level = level or (settings.log.level if settings else "INFO")
            logger.setLevel(getattr(logging, log_level.upper()))
        except AttributeError:
            # settingsがNoneの場合のフォールバック
            logger.setLevel(logging.INFO)
            log_level = "INFO"

        # フォーマッタの設定
        formatter = logging.Formatter(
            fmt="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
            datefmt="%Y-%m-%d %H:%M:%S",
        )

        # コンソールハンドラの設定
        console_handler = logging.StreamHandler(sys.stdout)
        console_handler.setLevel(logging.INFO)
        console_handler.setFormatter(formatter)
        logger.addHandler(console_handler)

        # ファイルハンドラの設定（settingsがある場合のみ）
        if log_file is None and settings:
            try:
                log_file = settings.logs_dir / settings.log.file.split("/")[-1]
            except AttributeError:
                # settingsの属性にアクセスできない場合はファイルハンドラをスキップ
                log_file = None

        if log_file is not None:
            try:
                log_file.parent.mkdir(parents=True, exist_ok=True)
            except OSError as e:
                raise LoggingError(f"Failed to create log directory: {log_file.parent}") from e

            try:
                max_bytes = settings.log.max_bytes if settings else 10485760
                backup_count = settings.log.backup_count if settings else 5

                file_handler = RotatingFileHandler(
                    log_file,
                    maxBytes=max_bytes,
                    backupCount=backup_count,
                    encoding="utf-8",
                )
                file_handler.setLevel(getattr(logging, log_level.upper()))
                file_handler.setFormatter(formatter)
                logger.addHandler(file_handler)
            except OSError as e:
                raise LoggingError(f"Failed to create log file handler: {log_file}") from e

        # プロパゲーションを無効化（重複ログ防止）
        logger.propagate = False

        return logger

    except Exception as e:
        if isinstance(e, LoggingError):
            raise
        raise LoggingError(f"Failed to setup logger '{name}'") from e


# ロガーキャッシュ
_logger_cache: dict[str, logging.Logger] = {}

# デフォルトロガー
default_logger = setup_logger("inheritance_calculator")
_logger_cache["inheritance_calculator"] = default_logger


def get_logger(name: str) -> logging.Logger:
    """
    指定された名前のロガーを取得します（キャッシュ付き）。

    Args:
        name: ロガー名

    Returns:
        ロガー
    """
    if name not in _logger_cache:
        _logger_cache[name] = setup_logger(name)
    return _logger_cache[name]



"""Neo4jServiceのユニットテスト

相続ケースのNeo4j保存機能のテスト
"""
import pytest
from datetime import date
from unittest.mock import Mock, MagicMock, patch, call
from neo4j.exceptions import Neo4jError

from inheritance_calculator_core.services.neo4j_service import Neo4jService
from inheritance_calculator_core.models.person import Person, Gender
from inheritance_calculator_core.models.relationship import BloodType
from inheritance_calculator_core.models.inheritance import InheritanceResult, Heir, HeritageRank
from inheritance_calculator_core.utils.exceptions import DatabaseException


class TestNeo4jService:
    """Neo4jServiceのユニットテスト"""

    @pytest.fixture
    def mock_client(self):
        """モッククライアントのフィクスチャ"""
        client = Mock()
        client.transaction.return_value.__enter__ = Mock(return_value=client)
        client.transaction.return_value.__exit__ = Mock(return_value=False)
        return client

    @pytest.fixture
    def neo4j_service(self, mock_client):
        """Neo4jServiceのフィクスチャ"""
        return Neo4jService(mock_client)

    @pytest.fixture
    def sample_decedent(self):
        """サンプル被相続人のフィクスチャ"""
        return Person(
            name="被相続人太郎",
            is_alive=False,
            is_decedent=True,
            death_date=date(2025, 6, 15),
            gender=Gender.MALE
        )

    @pytest.fixture
    def sample_spouse(self):
        """サンプル配偶者のフィクスチャ"""
        return Person(
            name="配偶者花子",
            is_alive=True,
            gender=Gender.FEMALE
        )

    @pytest.fixture
    def sample_children(self):
        """サンプル子リストのフィクスチャ"""
        return [
            Person(name="子一郎", is_alive=True, gender=Gender.MALE),
            Person(name="子二郎", is_alive=True, gender=Gender.MALE)
        ]

    @pytest.fixture
    def sample_result(self, sample_spouse, sample_children):
        """サンプル計算結果のフィクスチャ"""
        from fractions import Fraction

        heir1 = Heir(
            person=sample_spouse,
            rank=HeritageRank.SPOUSE,
            share=Fraction(1, 2),
            share_percentage=50.0
        )
        heir2 = Heir(
            person=sample_children[0],
            rank=HeritageRank.FIRST,
            share=Fraction(1, 4),
            share_percentage=25.0
        )
        heir3 = Heir(
            person=sample_children[1],
            rank=HeritageRank.FIRST,
            share=Fraction(1, 4),
            share_percentage=25.0
        )

        return InheritanceResult(
            decedent=Person(
                name="被相続人太郎",
                is_alive=False,
                is_decedent=True,
                death_date=date(2025, 6, 15)
            ),
            heirs=[heir1, heir2, heir3],
            calculation_basis=["民法第900条"]
        )

    def test_save_inheritance_case_with_spouse_and_children(
        self,
        neo4j_service,
        mock_client,
        sample_decedent,
        sample_spouse,
        sample_children,
        sample_result
    ):
        """配偶者と子がいるケースの保存テスト"""
        # PersonRepositoryとRelationshipRepositoryのモック設定
        with patch.object(neo4j_service.person_repo, 'create') as mock_create, \
             patch.object(neo4j_service.relationship_repo, 'create_spouse_of') as mock_spouse_rel, \
             patch.object(neo4j_service.relationship_repo, 'create_child_of') as mock_child_rel:

            neo4j_service.save_inheritance_case(
                decedent=sample_decedent,
                spouses=[sample_spouse],
                children=sample_children,
                parents=[],
                siblings=[],
                renounced=[],
                disqualified=[],
                disinherited=[],
                sibling_blood_types={},
                result=sample_result
            )

            # 被相続人、配偶者、子の作成が呼ばれたか確認
            assert mock_create.call_count == 4  # decedent + spouse + 2 children
            assert mock_spouse_rel.call_count == 1
            assert mock_child_rel.call_count == 2

    def test_save_inheritance_case_with_parents(
        self,
        neo4j_service,
        mock_client,
        sample_decedent,
        sample_result
    ):
        """直系尊属がいるケースの保存テスト"""
        parents = [
            Person(name="父", is_alive=True, gender=Gender.MALE),
            Person(name="母", is_alive=True, gender=Gender.FEMALE)
        ]

        with patch.object(neo4j_service.person_repo, 'create') as mock_create, \
             patch.object(neo4j_service.relationship_repo, 'create_child_of') as mock_child_rel:

            neo4j_service.save_inheritance_case(
                decedent=sample_decedent,
                spouses=[],
                children=[],
                parents=parents,
                siblings=[],
                renounced=[],
                disqualified=[],
                disinherited=[],
                sibling_blood_types={},
                result=sample_result
            )

            # 被相続人と親の作成が呼ばれたか確認
            assert mock_create.call_count == 3  # decedent + 2 parents
            # 親子関係は被相続人が子として登録される
            assert mock_child_rel.call_count == 2

    def test_save_inheritance_case_with_siblings(
        self,
        neo4j_service,
        mock_client,
        sample_decedent,
        sample_result
    ):
        """兄弟姉妹がいるケースの保存テスト"""
        brother = Person(name="兄", is_alive=True, gender=Gender.MALE)
        sister = Person(name="妹", is_alive=True, gender=Gender.FEMALE)
        siblings = [brother, sister]
        sibling_blood_types = {
            brother.id: BloodType.FULL,
            sister.id: BloodType.HALF
        }

        with patch.object(neo4j_service.person_repo, 'create') as mock_create, \
             patch.object(neo4j_service.relationship_repo, 'create_sibling_of') as mock_sibling_rel:

            neo4j_service.save_inheritance_case(
                decedent=sample_decedent,
                spouses=[],
                children=[],
                parents=[],
                siblings=siblings,
                renounced=[],
                disqualified=[],
                disinherited=[],
                sibling_blood_types=sibling_blood_types,
                result=sample_result
            )

            # 被相続人と兄弟姉妹の作成が呼ばれたか確認
            assert mock_create.call_count == 3  # decedent + 2 siblings
            assert mock_sibling_rel.call_count == 2

            # 血縁タイプが正しく渡されているか確認
            calls = mock_sibling_rel.call_args_list
            assert calls[0][1]['blood_type'] == BloodType.FULL
            assert calls[1][1]['blood_type'] == BloodType.HALF

    def test_save_inheritance_case_with_renounced(
        self,
        neo4j_service,
        mock_client,
        sample_decedent,
        sample_result
    ):
        """相続放棄者がいるケースの保存テスト"""
        renounced = [
            Person(name="放棄者", is_alive=True)
        ]

        with patch.object(neo4j_service.person_repo, 'create') as mock_create, \
             patch.object(neo4j_service.person_repo, 'find_by_name') as mock_find, \
             patch.object(neo4j_service.relationship_repo, 'create_renounced') as mock_renounced_rel:

            # 放棄者がまだ登録されていない場合
            mock_find.return_value = None

            neo4j_service.save_inheritance_case(
                decedent=sample_decedent,
                spouses=[],
                children=[],
                parents=[],
                siblings=[],
                renounced=renounced,
                disqualified=[],
                disinherited=[],
                sibling_blood_types={},
                result=sample_result
            )

            # 被相続人と放棄者の作成が呼ばれたか確認
            assert mock_create.call_count == 2  # decedent + renounced person
            assert mock_renounced_rel.call_count == 1

            # 放棄日が被相続人の死亡日になっているか確認
            call_args = mock_renounced_rel.call_args
            assert call_args[1]['renounce_date'] == sample_decedent.death_date

    def test_save_inheritance_case_with_existing_renounced_person(
        self,
        neo4j_service,
        mock_client,
        sample_decedent,
        sample_result
    ):
        """既に登録されている放棄者の保存テスト"""
        renounced = [
            Person(name="既存放棄者", is_alive=True)
        ]

        with patch.object(neo4j_service.person_repo, 'create') as mock_create, \
             patch.object(neo4j_service.person_repo, 'find_by_name') as mock_find, \
             patch.object(neo4j_service.relationship_repo, 'create_renounced') as mock_renounced_rel:

            # 放棄者が既に登録されている場合
            mock_find.return_value = renounced[0]

            neo4j_service.save_inheritance_case(
                decedent=sample_decedent,
                spouses=[],
                children=[],
                parents=[],
                siblings=[],
                renounced=renounced,
                disqualified=[],
                disinherited=[],
                sibling_blood_types={},
                result=sample_result
            )

            # 被相続人のみ作成（放棄者は既存なので作成されない）
            assert mock_create.call_count == 1  # decedent only
            # 放棄関係は作成される
            assert mock_renounced_rel.call_count == 1

    def test_save_inheritance_case_transaction_rollback_on_error(
        self,
        neo4j_service,
        mock_client,
        sample_decedent,
        sample_result
    ):
        """エラー発生時のトランザクションロールバックテスト"""
        with patch.object(neo4j_service.person_repo, 'create') as mock_create:
            # 2回目の作成でエラーを発生させる
            mock_create.side_effect = [None, DatabaseException("エラー")]

            with pytest.raises(DatabaseException) as exc_info:
                neo4j_service.save_inheritance_case(
                    decedent=sample_decedent,
                    spouses=[Person(name="配偶者", is_alive=True)],
                    children=[],
                    parents=[],
                    siblings=[],
                    renounced=[],
                    disqualified=[],
                    disinherited=[],
                    sibling_blood_types={},
                    result=sample_result
                )

            assert "Neo4j保存エラー" in str(exc_info.value)

    def test_save_inheritance_case_complex_scenario(
        self,
        neo4j_service,
        mock_client,
        sample_decedent,
        sample_spouse,
        sample_children,
        sample_result
    ):
        """複雑なケースの保存テスト（配偶者、子、親、兄弟、放棄者）"""
        parents = [Person(name="父", is_alive=True)]
        brother = Person(name="兄", is_alive=True)
        siblings = [brother]
        renounced = [Person(name="放棄者", is_alive=True)]
        sibling_blood_types = {brother.id: BloodType.FULL}

        with patch.object(neo4j_service.person_repo, 'create') as mock_create, \
             patch.object(neo4j_service.person_repo, 'find_by_name') as mock_find, \
             patch.object(neo4j_service.relationship_repo, 'create_spouse_of') as mock_spouse_rel, \
             patch.object(neo4j_service.relationship_repo, 'create_child_of') as mock_child_rel, \
             patch.object(neo4j_service.relationship_repo, 'create_sibling_of') as mock_sibling_rel, \
             patch.object(neo4j_service.relationship_repo, 'create_renounced') as mock_renounced_rel:

            mock_find.return_value = None

            neo4j_service.save_inheritance_case(
                decedent=sample_decedent,
                spouses=[sample_spouse],
                children=sample_children,
                parents=parents,
                siblings=siblings,
                renounced=renounced,
                disqualified=[],
                disinherited=[],
                sibling_blood_types=sibling_blood_types,
                result=sample_result
            )

            # 全ての人物が作成されたか確認
            # decedent + spouse + 2 children + parent + sibling + renounced = 7
            assert mock_create.call_count == 7

            # 全ての関係が作成されたか確認
            assert mock_spouse_rel.call_count == 1
            assert mock_child_rel.call_count == 3  # 2 children + 1 parent
            assert mock_sibling_rel.call_count == 1
            assert mock_renounced_rel.call_count == 1

    def test_clear_all_data(self, neo4j_service, mock_client):
        """全データ削除のテスト"""
        neo4j_service.clear_all_data()

        assert mock_client.clear_database.call_count == 1

    def test_save_inheritance_case_uses_transaction(
        self,
        neo4j_service,
        mock_client,
        sample_decedent,
        sample_result
    ):
        """トランザクションが使用されることの確認テスト"""
        with patch.object(neo4j_service.person_repo, 'create'):
            neo4j_service.save_inheritance_case(
                decedent=sample_decedent,
                spouses=[],
                children=[],
                parents=[],
                siblings=[],
                renounced=[],
                disqualified=[],
                disinherited=[],
                sibling_blood_types={},
                result=sample_result
            )

            # トランザクションが開始されたことを確認
            assert mock_client.transaction.call_count == 1

    def test_save_inheritance_case_empty_lists(
        self,
        neo4j_service,
        mock_client,
        sample_decedent,
        sample_result
    ):
        """空リストを渡した場合のテスト"""
        with patch.object(neo4j_service.person_repo, 'create') as mock_create:
            neo4j_service.save_inheritance_case(
                decedent=sample_decedent,
                spouses=[],
                children=[],
                parents=[],
                siblings=[],
                renounced=[],
                disqualified=[],
                disinherited=[],
                sibling_blood_types={},
                result=sample_result
            )

            # 被相続人のみ作成される
            assert mock_create.call_count == 1

    def test_save_inheritance_case_default_renounce_date(
        self,
        neo4j_service,
        mock_client,
        sample_result
    ):
        """被相続人の死亡日がNoneの場合の放棄日テスト"""
        decedent_no_death_date = Person(
            name="被相続人",
            is_alive=False,
            is_decedent=True,
            death_date=None  # 死亡日なし
        )
        renounced = [Person(name="放棄者", is_alive=True)]

        with patch.object(neo4j_service.person_repo, 'create'), \
             patch.object(neo4j_service.person_repo, 'find_by_name', return_value=None), \
             patch.object(neo4j_service.relationship_repo, 'create_renounced') as mock_renounced_rel, \
             patch('inheritance_calculator_core.services.neo4j_service.date') as mock_date:

            # date.today()をモック
            mock_today = date(2025, 10, 16)
            mock_date.today.return_value = mock_today

            neo4j_service.save_inheritance_case(
                decedent=decedent_no_death_date,
                spouses=[],
                children=[],
                parents=[],
                siblings=[],
                renounced=renounced,
                disqualified=[],
                disinherited=[],
                sibling_blood_types={},
                result=sample_result
            )

            # 放棄日がtoday()になっているか確認
            call_args = mock_renounced_rel.call_args
            assert call_args[1]['renounce_date'] == mock_today



"""相続情報収集インタビューエージェント

AIを使って対話形式で相続情報を収集するエージェント。
"""
from typing import Dict, List, Optional, Any
from datetime import date, datetime
from enum import Enum
import logging
import re

from .ollama_client import OllamaClient
from .prompts import InheritancePrompts
from ..models.person import Person, Gender
from ..models.relationship import BloodType


class InterviewState(str, Enum):
    """インタビューの状態"""
    INIT = "init"
    DECEDENT_INFO = "decedent_info"
    SPOUSE_INFO = "spouse_info"
    CHILDREN_INFO = "children_info"
    PARENTS_INFO = "parents_info"
    SIBLINGS_INFO = "siblings_info"
    SPECIAL_CASES = "special_cases"
    CONFIRMATION = "confirmation"
    COMPLETED = "completed"


class InterviewAgent:
    """
    対話型相続情報収集エージェント

    ユーザーと対話しながら相続に必要な情報を収集し、
    Personオブジェクトとして構造化する。
    """

    def __init__(self, ollama_client: Optional[OllamaClient] = None) -> None:
        """
        初期化

        Args:
            ollama_client: Ollamaクライアント（Noneの場合は新規作成）
        """
        self.logger = logging.getLogger(__name__)
        self.client = ollama_client or OllamaClient()
        self.prompts = InheritancePrompts()

        # 状態管理
        self.state = InterviewState.INIT
        self.collected_data: Dict[str, Any] = {}
        self.conversation_history: List[Dict[str, str]] = []

        # 収集したデータ
        self.decedent: Optional[Person] = None
        self.spouses: List[Person] = []
        self.children: List[Person] = []
        self.parents: List[Person] = []
        self.siblings: List[Person] = []
        self.renounced: List[Person] = []
        self.disqualified: List[Person] = []
        self.disinherited: List[Person] = []
        self.sibling_blood_types: Dict[str, BloodType] = {}
        self.retransfer_heirs_info: Dict[str, List[Person]] = {}

        self.logger.info("InterviewAgent initialized")

    def start_interview(self) -> str:
        """
        インタビューを開始

        Returns:
            最初の質問メッセージ
        """
        self.state = InterviewState.DECEDENT_INFO
        self.logger.info("Interview started")

        welcome_message = self.prompts.DECEDENT_INTRO + "\n\n" + self.prompts.DECEDENT_NAME

        # 会話履歴に追加
        self.conversation_history.append({
            "role": "assistant",
            "content": welcome_message
        })

        return welcome_message

    def process_response(self, user_input: str) -> str:
        """
        ユーザーの応答を処理し、次の質問を返す

        Args:
            user_input: ユーザーの入力

        Returns:
            次の質問または確認メッセージ
        """
        # 会話履歴に追加
        self.conversation_history.append({
            "role": "user",
            "content": user_input
        })

        try:
            # 状態に応じて処理を分岐
            if self.state == InterviewState.DECEDENT_INFO:
                next_message = self._process_decedent_info(user_input)
            elif self.state == InterviewState.SPOUSE_INFO:
                next_message = self._process_spouse_info(user_input)
            elif self.state == InterviewState.CHILDREN_INFO:
                next_message = self._process_children_info(user_input)
            elif self.state == InterviewState.PARENTS_INFO:
                next_message = self._process_parents_info(user_input)
            elif self.state == InterviewState.SIBLINGS_INFO:
                next_message = self._process_siblings_info(user_input)
            elif self.state == InterviewState.SPECIAL_CASES:
                next_message = self._process_special_cases(user_input)
            elif self.state == InterviewState.CONFIRMATION:
                next_message = self._process_confirmation(user_input)
            else:
                next_message = "インタビューは完了しました。"

            # 会話履歴に追加
            self.conversation_history.append({
                "role": "assistant",
                "content": next_message
            })

            return next_message

        except Exception as e:
            self.logger.error(f"Error processing response: {e}", exc_info=True)
            error_message = "申し訳ございません。エラーが発生しました。もう一度お答えいただけますか？"
            self.conversation_history.append({
                "role": "assistant",
                "content": error_message
            })
            return error_message

    def _process_decedent_info(self, user_input: str) -> str:
        """被相続人情報を処理"""
        # まだ名前を収集していない
        if "decedent_name" not in self.collected_data:
            self.collected_data["decedent_name"] = user_input.strip()
            return self.prompts.DECEDENT_DEATH_DATE

        # まだ死亡日を収集していない
        if "decedent_death_date" not in self.collected_data:
            death_date = self._parse_date(user_input)
            if death_date:
                self.collected_data["decedent_death_date"] = death_date
                return self.prompts.DECEDENT_BIRTH_DATE
            else:
                return "申し訳ございません。日付の形式が正しくありません。\n" + self.prompts.DECEDENT_DEATH_DATE

        # まだ生年月日を収集していない
        if "decedent_birth_date" not in self.collected_data:
            if user_input.strip() in ["不明", "わからない", "分からない"]:
                birth_date = None
            else:
                birth_date = self._parse_date(user_input)

            self.collected_data["decedent_birth_date"] = birth_date

            # 被相続人オブジェクトを作成
            self.decedent = Person(
                name=self.collected_data["decedent_name"],
                is_decedent=True,
                is_alive=False,
                death_date=self.collected_data["decedent_death_date"],
                birth_date=birth_date
            )

            # 次の状態へ
            self.state = InterviewState.SPOUSE_INFO
            return "\n" + self.prompts.SPOUSE_QUESTION

        return "エラーが発生しました。"

    def _process_spouse_info(self, user_input: str) -> str:
        """配偶者情報を処理"""
        # 配偶者の有無を確認
        if "has_spouse" not in self.collected_data:
            has_spouse = self._parse_yes_no(user_input)
            self.collected_data["has_spouse"] = has_spouse

            if has_spouse:
                return self.prompts.SPOUSE_INFO
            else:
                # 次の状態へ
                self.state = InterviewState.CHILDREN_INFO
                return "\n" + self.prompts.CHILDREN_QUESTION

        # 配偶者情報を収集中
        # LLMで構造化データ抽出
        spouse_info = self._extract_person_info(user_input, "配偶者")
        spouse = Person(
            name=spouse_info.get("name", "配偶者"),
            is_alive=spouse_info.get("is_alive", True),
            birth_date=spouse_info.get("birth_date"),
            gender=spouse_info.get("gender", Gender.UNKNOWN)
        )
        self.spouses.append(spouse)

        # 次の状態へ
        self.state = InterviewState.CHILDREN_INFO
        return "\n" + self.prompts.CHILDREN_QUESTION

    def _process_children_info(self, user_input: str) -> str:
        """子の情報を処理"""
        if "has_children" not in self.collected_data:
            has_children = self._parse_yes_no(user_input)
            self.collected_data["has_children"] = has_children

            if has_children:
                return self.prompts.CHILDREN_COUNT
            else:
                # 次の状態へ
                self.state = InterviewState.PARENTS_INFO
                return "\n" + self.prompts.PARENTS_QUESTION

        # 子の人数を確認
        if "children_count" not in self.collected_data:
            try:
                count = int(user_input.strip())
                self.collected_data["children_count"] = count
                self.collected_data["children_collected"] = 0
                return self.prompts.format_child_info(1)
            except ValueError:
                return "申し訳ございません。数字でご入力ください。\n" + self.prompts.CHILDREN_COUNT

        # 子の情報を収集中
        collected = self.collected_data["children_collected"]
        total = self.collected_data["children_count"]

        # LLMで構造化データ抽出
        child_info = self._extract_person_info(user_input, f"子{collected + 1}")
        child = Person(
            name=child_info.get("name", f"子{collected + 1}"),
            is_alive=child_info.get("is_alive", True),
            birth_date=child_info.get("birth_date"),
            death_date=child_info.get("death_date"),
            gender=child_info.get("gender", Gender.UNKNOWN)
        )
        self.children.append(child)

        collected += 1
        self.collected_data["children_collected"] = collected

        if collected < total:
            return self.prompts.format_child_info(collected + 1)
        else:
            # 次の状態へ
            self.state = InterviewState.PARENTS_INFO
            return "\n" + self.prompts.PARENTS_QUESTION

    def _process_parents_info(self, user_input: str) -> str:
        """直系尊属情報を処理"""
        if "has_parents" not in self.collected_data:
            has_parents = self._parse_yes_no(user_input)
            self.collected_data["has_parents"] = has_parents

            if not has_parents:
                # 次の状態へ
                self.state = InterviewState.SIBLINGS_INFO
                return "\n" + self.prompts.SIBLINGS_QUESTION

            # 親の人数を聞く
            self.collected_data["parents"] = []
            return "存命の直系尊属（父母、祖父母など）は何人いますか？"

        # 親の情報を収集
        if "parent_count" not in self.collected_data:
            try:
                count = int(user_input.strip())
                self.collected_data["parent_count"] = count
                self.collected_data["parent_collected"] = 0

                if count == 0:
                    self.state = InterviewState.SIBLINGS_INFO
                    return "\n" + self.prompts.SIBLINGS_QUESTION

                return f"1人目の直系尊属について教えてください。\n氏名、続柄（父、母、祖父など）、生年月日などを入力してください。"
            except ValueError:
                return "数字で入力してください。存命の直系尊属は何人いますか？"

        # 親の詳細情報を収集
        collected = self.collected_data["parent_collected"]
        count = self.collected_data["parent_count"]

        parent_info = self._extract_person_info(user_input, f"直系尊属{collected + 1}")
        parent = Person(
            name=parent_info.get("name", f"直系尊属{collected + 1}"),
            is_alive=True,  # 存命の直系尊属のみ
            birth_date=parent_info.get("birth_date"),
            gender=parent_info.get("gender", Gender.UNKNOWN)
        )
        self.collected_data["parents"].append(parent)
        self.collected_data["parent_collected"] = collected + 1

        if collected + 1 < count:
            return f"{collected + 2}人目の直系尊属について教えてください。\n氏名、続柄、生年月日などを入力してください。"

        # 全員収集完了
        self.state = InterviewState.SIBLINGS_INFO
        return "\n" + self.prompts.SIBLINGS_QUESTION

    def _process_siblings_info(self, user_input: str) -> str:
        """兄弟姉妹情報を処理"""
        if "has_siblings" not in self.collected_data:
            has_siblings = self._parse_yes_no(user_input)
            self.collected_data["has_siblings"] = has_siblings

            if not has_siblings:
                # 次の状態へ（特殊ケース）
                self.state = InterviewState.SPECIAL_CASES
                self.collected_data["special_case_step"] = "renunciation"
                return "\n" + self.prompts.RENUNCIATION_QUESTION

            # 兄弟姉妹の人数を聞く
            self.collected_data["siblings"] = []
            return "存命の兄弟姉妹は何人いますか？"

        # 兄弟姉妹の情報を収集
        if "sibling_count" not in self.collected_data:
            try:
                count = int(user_input.strip())
                self.collected_data["sibling_count"] = count
                self.collected_data["sibling_collected"] = 0

                if count == 0:
                    self.state = InterviewState.SPECIAL_CASES
                    self.collected_data["special_case_step"] = "renunciation"
                    return "\n" + self.prompts.RENUNCIATION_QUESTION

                return f"1人目の兄弟姉妹について教えてください。\n氏名、続柄（兄、姉、弟、妹など）、血縁タイプ（全血・半血）、生年月日などを入力してください。"
            except ValueError:
                return "数字で入力してください。存命の兄弟姉妹は何人いますか？"

        # 兄弟姉妹の詳細情報を収集
        collected = self.collected_data["sibling_collected"]
        count = self.collected_data["sibling_count"]

        sibling_info = self._extract_person_info(user_input, f"兄弟姉妹{collected + 1}")
        sibling = Person(
            name=sibling_info.get("name", f"兄弟姉妹{collected + 1}"),
            is_alive=True,  # 存命の兄弟姉妹のみ
            birth_date=sibling_info.get("birth_date"),
            gender=sibling_info.get("gender", Gender.UNKNOWN)
        )
        self.collected_data["siblings"].append(sibling)
        self.collected_data["sibling_collected"] = collected + 1

        if collected + 1 < count:
            return f"{collected + 2}人目の兄弟姉妹について教えてください。\n氏名、続柄、血縁タイプ、生年月日などを入力してください。"

        # 全員収集完了
        self.state = InterviewState.SPECIAL_CASES
        self.collected_data["special_case_step"] = "renunciation"
        return "\n" + self.prompts.RENUNCIATION_QUESTION

    def _process_special_cases(self, user_input: str) -> str:
        """特殊ケース（相続放棄等）を処理"""
        step = self.collected_data.get("special_case_step", "renunciation")

        if step == "renunciation":
            has_renunciation = self._parse_yes_no(user_input)
            self.collected_data["has_renunciation"] = has_renunciation

            # 次のステップへ
            self.collected_data["special_case_step"] = "retransfer"
            return "\n" + self.prompts.RETRANSFER_QUESTION

        if step == "retransfer":
            has_retransfer = self._parse_yes_no(user_input)
            self.collected_data["has_retransfer"] = has_retransfer

            # 確認へ
            self.state = InterviewState.CONFIRMATION
            summary = self._generate_summary()
            return "\n" + self.prompts.format_confirmation(summary)

        return "エラーが発生しました。"

    def _process_confirmation(self, user_input: str) -> str:
        """確認を処理"""
        confirmed = self._parse_yes_no(user_input)

        if confirmed:
            self.state = InterviewState.COMPLETED
            return self.prompts.CALCULATION_START
        else:
            return "修正が必要な項目を教えてください。"

    def _parse_yes_no(self, text: str) -> bool:
        """はい/いいえを解析"""
        text = text.strip().lower()
        if text in ["はい", "yes", "y", "有", "あり", "います", "いる"]:
            return True
        return False

    def _extract_person_info(self, user_input: str, role: str) -> Dict[str, Any]:
        """
        ユーザー入力から人物情報を抽出

        Args:
            user_input: ユーザーの入力テキスト
            role: 役割（配偶者、子1、など）

        Returns:
            抽出した人物情報の辞書
        """
        # LLMを使って構造化データ抽出
        extraction_prompt = f"""
以下のテキストから{role}の情報を抽出してください。
抽出する情報:
- 氏名（name）
- 生存状態（is_alive: true/false）
- 生年月日（birth_date: YYYY-MM-DD形式、不明な場合はnull）
- 死亡日（death_date: YYYY-MM-DD形式、存命または不明な場合はnull）
- 性別（gender: male/female/other/unknown）

テキスト: {user_input}

以下のJSON形式で回答してください:
{{"name": "氏名", "is_alive": true, "birth_date": "YYYY-MM-DD", "death_date": null, "gender": "unknown"}}
"""

        try:
            messages = [
                {"role": "system", "content": "あなたは相続情報を正確に抽出するアシスタントです。"},
                {"role": "user", "content": extraction_prompt}
            ]

            response = self.client.chat(messages, temperature=0.1)

            # JSON部分を抽出
            import json
            json_match = re.search(r'\{[^}]+\}', response)
            if json_match:
                extracted: Dict[str, Any] = json.loads(json_match.group())

                # 日付をdateオブジェクトに変換
                if extracted.get("birth_date"):
                    extracted["birth_date"] = self._parse_date(extracted["birth_date"])
                if extracted.get("death_date"):
                    extracted["death_date"] = self._parse_date(extracted["death_date"])

                # 性別をGenderに変換
                if extracted.get("gender"):
                    gender_map = {
                        "male": Gender.MALE,
                        "female": Gender.FEMALE,
                        "other": Gender.OTHER,
                        "unknown": Gender.UNKNOWN
                    }
                    extracted["gender"] = gender_map.get(extracted["gender"].lower(), Gender.UNKNOWN)

                return extracted

        except Exception as e:
            self.logger.warning(f"Failed to extract person info with LLM: {e}")

        # フォールバック: 簡易パース
        return self._simple_parse_person_info(user_input, role)

    def _simple_parse_person_info(self, user_input: str, role: str) -> Dict[str, Any]:
        """
        簡易的な人物情報パース（フォールバック用）

        Args:
            user_input: ユーザー入力
            role: 役割

        Returns:
            抽出した情報
        """
        lines = user_input.strip().split("\n")
        name = lines[0] if lines else role

        # 生存状態の推測
        is_alive = "死亡" not in user_input and "亡くなっ" not in user_input

        # 日付の抽出
        birth_date = None
        death_date = None
        for line in lines:
            if "生年月日" in line or "生まれ" in line:
                birth_date = self._parse_date(line)
            if "死亡" in line or "亡くなっ" in line:
                death_date = self._parse_date(line)
                is_alive = False

        return {
            "name": name,
            "is_alive": is_alive,
            "birth_date": birth_date,
            "death_date": death_date,
            "gender": Gender.UNKNOWN
        }

    def _parse_date(self, text: str) -> Optional[date]:
        """日付文字列を解析"""
        text = text.strip()

        # YYYY-MM-DD形式
        match = re.match(r'(\d{4})-(\d{1,2})-(\d{1,2})', text)
        if match:
            try:
                return date(int(match.group(1)), int(match.group(2)), int(match.group(3)))
            except ValueError:
                pass

        # YYYY/MM/DD形式
        match = re.match(r'(\d{4})/(\d{1,2})/(\d{1,2})', text)
        if match:
            try:
                return date(int(match.group(1)), int(match.group(2)), int(match.group(3)))
            except ValueError:
                pass

        # YYYY年MM月DD日形式
        match = re.match(r'(\d{4})年(\d{1,2})月(\d{1,2})日', text)
        if match:
            try:
                return date(int(match.group(1)), int(match.group(2)), int(match.group(3)))
            except ValueError:
                pass

        return None

    def _generate_summary(self) -> str:
        """収集した情報のサマリーを生成"""
        lines = []
        lines.append(f"被相続人: {self.decedent.name if self.decedent else '不明'}")
        if self.decedent and self.decedent.death_date:
            lines.append(f"死亡日: {self.decedent.death_date}")

        if self.spouses:
            lines.append(f"\n配偶者: {', '.join(s.name for s in self.spouses)}")

        if self.children:
            lines.append(f"\n子: {', '.join(c.name for c in self.children)}")

        if self.parents:
            lines.append(f"\n直系尊属: {', '.join(p.name for p in self.parents)}")

        if self.siblings:
            lines.append(f"\n兄弟姉妹: {', '.join(s.name for s in self.siblings)}")

        return "\n".join(lines)

    def get_collected_data(self) -> Dict[str, Any]:
        """
        収集したデータを取得

        Returns:
            相続計算に必要なデータ
        """
        return {
            "decedent": self.decedent,
            "spouses": self.spouses,
            "children": self.children,
            "parents": self.parents,
            "siblings": self.siblings,
            "renounced": self.renounced,
            "disqualified": self.disqualified,
            "disinherited": self.disinherited,
            "sibling_blood_types": self.sibling_blood_types,
            "retransfer_heirs_info": self.retransfer_heirs_info,
        }

    def is_completed(self) -> bool:
        """インタビューが完了したか確認"""
        return self.state == InterviewState.COMPLETED



"""再転相続のテスト"""
from datetime import date
from fractions import Fraction

import pytest

from inheritance_calculator_core.models.person import Person
from inheritance_calculator_core.services.inheritance_calculator import InheritanceCalculator


class TestRetransferInheritance:
    """再転相続のテストクラス"""

    def test_retransfer_basic(self) -> None:
        """基本的な再転相続のテスト（配偶者1人・子2人）

        ケース8: 再転相続
        - 被相続人: A（2025年1月死亡）
        - 子B（2025年2月死亡、遺産分割前）
        - Bの相続人: 配偶者C、子D、子E
        - 期待結果（民法900条1号に基づく）:
          - C（配偶者）: 1/1 × 1/2 = 1/2
          - D（子）: 1/1 × 1/4 = 1/4
          - E（子）: 1/1 × 1/4 = 1/4
        """
        # 被相続人A
        decedent = Person(
            name="A",
            is_decedent=True,
            is_alive=False,
            death_date=date(2025, 1, 15)
        )

        # 子B（遺産分割前に死亡）
        child_b = Person(
            name="B",
            is_alive=False,
            death_date=date(2025, 2, 10),
            died_before_division=True  # 遺産分割前に死亡
        )

        # Bの配偶者C
        spouse_c = Person(
            name="C",
            is_alive=True
        )

        # Bの子D
        child_d = Person(
            name="D",
            is_alive=True
        )

        # Bの子E
        child_e = Person(
            name="E",
            is_alive=True
        )

        calculator = InheritanceCalculator()

        # 再転相続先の情報を設定（配偶者1人・子2人）
        retransfer_info = {
            child_b.id: [spouse_c, child_d, child_e]
        }

        # 再転相続先の関係情報を設定
        retransfer_relationships = {
            child_b.id: {
                spouse_c.id: 'spouse',
                child_d.id: 'child',
                child_e.id: 'child'
            }
        }

        result = calculator.calculate(
            decedent=decedent,
            spouses=[],
            children=[child_b],
            parents=[],
            siblings=[],
            retransfer_heirs_info=retransfer_info,
            retransfer_relationships=retransfer_relationships
        )

        # 検証
        assert result.total_heirs == 3
        assert "再転相続" in str(result.calculation_basis) or "民法第896条" in str(result.calculation_basis)

        # 法定相続分に基づく按分（均等分割ではない）
        heirs_by_name = {heir.person.name: heir for heir in result.heirs}
        assert "C" in heirs_by_name
        assert "D" in heirs_by_name
        assert "E" in heirs_by_name

        # 配偶者: 1/2、子: 各1/4（民法900条1号）
        assert heirs_by_name["C"].share == Fraction(1, 2)
        assert heirs_by_name["D"].share == Fraction(1, 4)
        assert heirs_by_name["E"].share == Fraction(1, 4)

        assert heirs_by_name["C"].is_retransfer is True
        assert heirs_by_name["D"].is_retransfer is True
        assert heirs_by_name["E"].is_retransfer is True

    def test_retransfer_with_spouse_and_retransfer_heir(self) -> None:
        """配偶者と再転相続が混在するケース"""
        # 被相続人A
        decedent = Person(
            name="A",
            is_decedent=True,
            is_alive=False,
            death_date=date(2025, 1, 15)
        )

        # 配偶者（存命）
        spouse = Person(
            name="配偶者",
            is_alive=True
        )

        # 子B（遺産分割前に死亡）
        child_b = Person(
            name="B",
            is_alive=False,
            death_date=date(2025, 2, 10),
            died_before_division=True
        )

        # Bの子C
        grandchild_c = Person(
            name="C（Bの子）",
            is_alive=True
        )

        calculator = InheritanceCalculator()

        # 再転相続先の情報
        retransfer_info = {
            child_b.id: [grandchild_c]
        }

        result = calculator.calculate(
            decedent=decedent,
            spouses=[spouse],
            children=[child_b],
            parents=[],
            siblings=[],
            retransfer_heirs_info=retransfer_info
        )

        # 検証: 配偶者1/2、子Bの分1/2がCに再転相続
        assert result.total_heirs == 2
        heirs_by_name = {heir.person.name: heir for heir in result.heirs}

        assert "配偶者" in heirs_by_name
        assert heirs_by_name["配偶者"].share == Fraction(1, 2)
        assert heirs_by_name["配偶者"].is_retransfer is False

        assert "C（Bの子）" in heirs_by_name
        assert heirs_by_name["C（Bの子）"].share == Fraction(1, 2)
        assert heirs_by_name["C（Bの子）"].is_retransfer is True

    def test_no_retransfer_when_heir_alive(self) -> None:
        """相続人が存命の場合は再転相続が発生しない"""
        decedent = Person(
            name="A",
            is_decedent=True,
            is_alive=False
        )

        # 存命の子
        child = Person(
            name="B",
            is_alive=True,
            died_before_division=False
        )

        calculator = InheritanceCalculator()

        result = calculator.calculate(
            decedent=decedent,
            spouses=[],
            children=[child],
            parents=[],
            siblings=[]
        )

        # 検証: 再転相続は発生しない
        assert result.total_heirs == 1
        assert result.heirs[0].person.name == "B"
        assert result.heirs[0].is_retransfer is False
        assert "再転相続" not in str(result.calculation_basis)
        assert "民法第896条" not in str(result.calculation_basis)

    def test_retransfer_multiple_heirs(self) -> None:
        """複数の相続人が遺産分割前に死亡したケース"""
        decedent = Person(
            name="A",
            is_decedent=True,
            is_alive=False,
            death_date=date(2025, 1, 15)
        )

        # 子B（遺産分割前に死亡）
        child_b = Person(
            name="B",
            is_alive=False,
            death_date=date(2025, 2, 10),
            died_before_division=True
        )

        # 子C（遺産分割前に死亡）
        child_c = Person(
            name="C",
            is_alive=False,
            death_date=date(2025, 2, 20),
            died_before_division=True
        )

        # Bの子D
        grandchild_d = Person(
            name="D（Bの子）",
            is_alive=True
        )

        # Cの子E
        grandchild_e = Person(
            name="E（Cの子）",
            is_alive=True
        )

        calculator = InheritanceCalculator()

        # 再転相続先の情報
        retransfer_info = {
            child_b.id: [grandchild_d],
            child_c.id: [grandchild_e]
        }

        result = calculator.calculate(
            decedent=decedent,
            spouses=[],
            children=[child_b, child_c],
            parents=[],
            siblings=[],
            retransfer_heirs_info=retransfer_info
        )

        # 検証: DとEがそれぞれ1/2ずつ相続
        assert result.total_heirs == 2
        heirs_by_name = {heir.person.name: heir for heir in result.heirs}

        assert "D（Bの子）" in heirs_by_name
        assert "E（Cの子）" in heirs_by_name
        assert heirs_by_name["D（Bの子）"].share == Fraction(1, 2)
        assert heirs_by_name["E（Cの子）"].share == Fraction(1, 2)
        assert heirs_by_name["D（Bの子）"].is_retransfer is True
        assert heirs_by_name["E（Cの子）"].is_retransfer is True



"""人物モデル"""
from datetime import date
from typing import Optional, Any
from enum import Enum

from pydantic import Field, field_validator, EmailStr

from .base import Neo4jNode


class Gender(str, Enum):
    """性別"""
    MALE = "male"
    FEMALE = "female"
    OTHER = "other"
    UNKNOWN = "unknown"


class Person(Neo4jNode):
    """
    人物モデル

    被相続人および相続人候補を表現するモデル。
    Neo4jのPersonノードに対応。
    """

    name: str = Field(..., description="氏名", min_length=1)
    is_alive: bool = Field(default=True, description="生存しているか")
    death_date: Optional[date] = Field(default=None, description="死亡日")
    is_decedent: bool = Field(default=False, description="被相続人フラグ")
    birth_date: Optional[date] = Field(default=None, description="生年月日")
    gender: Gender = Field(default=Gender.UNKNOWN, description="性別")
    died_before_division: bool = Field(
        default=False,
        description="遺産分割前に死亡したか（再転相続の対象）"
    )

    # 連絡先情報（オプショナル）
    address: Optional[str] = Field(default=None, description="住所")
    phone: Optional[str] = Field(default=None, description="電話番号")
    email: Optional[EmailStr] = Field(default=None, description="メールアドレス")

    @field_validator('death_date')
    @classmethod
    def validate_death_date(cls, v: Optional[date], info: Any) -> Optional[date]:
        """死亡日の検証"""
        if v is not None:
            # 死亡日がある場合、is_aliveはFalseであるべき
            # ただし、バリデーション時点ではis_aliveがまだ設定されていない可能性がある
            pass
        return v

    @field_validator('birth_date')
    @classmethod
    def validate_birth_date(cls, v: Optional[date], info: Any) -> Optional[date]:
        """生年月日の検証"""
        if v is not None:
            # 未来の日付は不可
            from datetime import date as date_class
            if v > date_class.today():
                raise ValueError("Birth date cannot be in the future")
        return v

    def mark_as_deceased(self, death_date: date) -> None:
        """
        死亡として記録

        Args:
            death_date: 死亡日
        """
        self.is_alive = False
        self.death_date = death_date
        self.mark_updated()

    def mark_as_decedent(self) -> None:
        """被相続人として記録"""
        self.is_decedent = True
        self.mark_updated()

    def set_contact_info(
        self,
        address: Optional[str] = None,
        phone: Optional[str] = None,
        email: Optional[str] = None
    ) -> None:
        """
        連絡先情報を設定

        Args:
            address: 住所
            phone: 電話番号
            email: メールアドレス
        """
        if address is not None:
            self.address = address
        if phone is not None:
            self.phone = phone
        if email is not None:
            self.email = email
        self.mark_updated()

    @property
    def age_at_death(self) -> Optional[int]:
        """
        死亡時の年齢を取得

        Returns:
            死亡時の年齢（生年月日または死亡日が不明な場合はNone）
        """
        if self.birth_date is None or self.death_date is None:
            return None

        age = self.death_date.year - self.birth_date.year
        # 誕生日前なら1歳引く
        if (self.death_date.month, self.death_date.day) < (self.birth_date.month, self.birth_date.day):
            age -= 1

        return age

    @property
    def current_age(self) -> Optional[int]:
        """
        現在の年齢を取得

        Returns:
            現在の年齢（生年月日が不明な場合はNone）
        """
        if self.birth_date is None:
            return None

        if not self.is_alive and self.death_date is not None:
            return self.age_at_death

        from datetime import date as date_class
        today = date_class.today()
        age = today.year - self.birth_date.year
        # 誕生日前なら1歳引く
        if (today.month, today.day) < (self.birth_date.month, self.birth_date.day):
            age -= 1

        return age

    def is_older_than(self, other: 'Person') -> bool:
        """
        指定された人物より年上かどうかを判定

        Args:
            other: 比較対象の人物

        Returns:
            年上の場合True（生年月日が不明な場合はFalse）
        """
        if self.birth_date is None or other.birth_date is None:
            return False

        return self.birth_date < other.birth_date

    def died_before(self, other: 'Person') -> bool:
        """
        指定された人物より先に死亡したかどうかを判定

        Args:
            other: 比較対象の人物

        Returns:
            先に死亡している場合True
        """
        if self.death_date is None:
            return False

        if other.death_date is None:
            # 自分は死亡、相手は生存 → 自分が先
            return True

        return self.death_date < other.death_date

    def __str__(self) -> str:
        """文字列表現"""
        status = "故人" if not self.is_alive else "存命"
        decedent_mark = "（被相続人）" if self.is_decedent else ""

        # 年齢情報の追加
        age_info = ""
        if self.current_age is not None:
            if self.is_alive:
                age_info = f", {self.current_age}歳"
            else:
                age_info = f", 享年{self.current_age}歳"

        return f"{self.name} ({status}{age_info}){decedent_mark}"

    def __repr__(self) -> str:
        """デバッグ用文字列表現"""
        return f"Person(name='{self.name}', is_alive={self.is_alive}, is_decedent={self.is_decedent})"



"""Services package exports"""
from .factory import ServiceFactory
from .heir_validator import HeirValidator
from .share_calculator import ShareCalculator
from .inheritance_calculator import InheritanceCalculator

__all__ = [
    "ServiceFactory",
    "HeirValidator",
    "ShareCalculator",
    "InheritanceCalculator",
]



"""HeirValidatorのユニットテスト

相続人資格検証サービスのテスト
"""
import pytest
from datetime import date

from inheritance_calculator_core.services.heir_validator import HeirValidator
from inheritance_calculator_core.models.person import Person, Gender
from inheritance_calculator_core.models.inheritance import HeritageRank, SubstitutionType


class TestHeirValidator:
    """HeirValidator のユニットテスト"""

    @pytest.fixture
    def validator(self):
        """バリデーターのフィクスチャ"""
        return HeirValidator()

    @pytest.fixture
    def sample_decedent(self):
        """サンプル被相続人のフィクスチャ"""
        return Person(
            name="被相続人太郎",
            is_alive=False,
            is_decedent=True,
            death_date=date(2025, 6, 15),
            gender=Gender.MALE
        )

    def test_set_decedent_valid(self, validator, sample_decedent):
        """被相続人設定の正常テスト"""
        validator.set_decedent(sample_decedent)

        assert validator.decedent == sample_decedent

    def test_set_decedent_without_is_decedent_flag(self, validator):
        """is_decedentフラグなしの被相続人設定テスト（警告が出る）"""
        person = Person(
            name="太郎",
            is_alive=False,
            is_decedent=False,  # フラグがFalse
            death_date=date(2025, 6, 15)
        )

        validator.set_decedent(person)
        assert validator.decedent == person

    def test_is_valid_heir_without_decedent(self, validator):
        """被相続人未設定での検証テスト"""
        person = Person(name="太郎", is_alive=True)

        with pytest.raises(ValueError) as exc_info:
            validator.is_valid_heir(person)

        assert "Decedent must be set" in str(exc_info.value)

    def test_is_valid_heir_same_as_decedent(self, validator, sample_decedent):
        """被相続人本人は相続人になれないテスト"""
        validator.set_decedent(sample_decedent)

        result = validator.is_valid_heir(sample_decedent)

        assert result is False

    def test_is_valid_heir_dead_person_not_before_division(self, validator, sample_decedent):
        """遺産分割前に死亡していない死者は相続人になれないテスト"""
        validator.set_decedent(sample_decedent)

        dead_person = Person(
            name="死亡者",
            is_alive=False,
            death_date=date(2020, 1, 1)
        )

        result = validator.is_valid_heir(dead_person)

        assert result is False

    def test_is_valid_heir_dead_before_division(self, validator, sample_decedent):
        """遺産分割前に死亡した人は一旦有効（再転相続対象）テスト"""
        validator.set_decedent(sample_decedent)

        dead_before_division = Person(
            name="遺産分割前死亡者",
            is_alive=False,
            death_date=date(2025, 7, 1),  # 被相続人死亡後
            died_before_division=True
        )

        result = validator.is_valid_heir(dead_before_division)

        assert result is True

    def test_is_valid_heir_renounced(self, validator, sample_decedent):
        """相続放棄者は相続人になれないテスト"""
        validator.set_decedent(sample_decedent)

        renounced_person = Person(name="放棄者", is_alive=True)
        validator.renounced_persons = [renounced_person]

        result = validator.is_valid_heir(renounced_person)

        assert result is False

    def test_is_valid_heir_disqualified(self, validator, sample_decedent):
        """相続欠格者は相続人になれないテスト"""
        validator.set_decedent(sample_decedent)

        disqualified_person = Person(name="欠格者", is_alive=True)
        validator.disqualified_persons = [disqualified_person]

        result = validator.is_valid_heir(disqualified_person)

        assert result is False

    def test_is_valid_heir_disinherited(self, validator, sample_decedent):
        """相続廃除者は相続人になれないテスト"""
        validator.set_decedent(sample_decedent)

        disinherited_person = Person(name="廃除者", is_alive=True)
        validator.disinherited_persons = [disinherited_person]

        result = validator.is_valid_heir(disinherited_person)

        assert result is False

    def test_validate_spouse_valid(self, validator, sample_decedent):
        """配偶者の検証テスト（有効）"""
        validator.set_decedent(sample_decedent)

        spouse = Person(name="配偶者花子", is_alive=True, gender=Gender.FEMALE)

        result = validator.validate_spouse(spouse)

        assert result is True

    def test_validate_spouse_invalid(self, validator, sample_decedent):
        """配偶者の検証テスト（無効・放棄者）"""
        validator.set_decedent(sample_decedent)

        spouse = Person(name="放棄配偶者", is_alive=True)
        validator.renounced_persons = [spouse]

        result = validator.validate_spouse(spouse)

        assert result is False

    def test_validate_child_valid(self, validator, sample_decedent):
        """子の検証テスト（有効）"""
        validator.set_decedent(sample_decedent)

        child = Person(name="子一郎", is_alive=True, gender=Gender.MALE)

        result = validator.validate_child(child)

        assert result is True

    def test_validate_child_invalid(self, validator, sample_decedent):
        """子の検証テスト（無効・死亡）"""
        validator.set_decedent(sample_decedent)

        dead_child = Person(
            name="死亡した子",
            is_alive=False,
            death_date=date(2020, 1, 1)
        )

        result = validator.validate_child(dead_child)

        assert result is False

    def test_validate_parent_valid_no_first_rank(self, validator, sample_decedent):
        """直系尊属の検証テスト（有効・第1順位なし）"""
        validator.set_decedent(sample_decedent)

        parent = Person(name="父", is_alive=True, gender=Gender.MALE)

        result = validator.validate_parent(parent, has_first_rank=False)

        assert result is True

    def test_validate_parent_invalid_has_first_rank(self, validator, sample_decedent):
        """直系尊属の検証テスト（無効・第1順位あり）"""
        validator.set_decedent(sample_decedent)

        parent = Person(name="母", is_alive=True, gender=Gender.FEMALE)

        result = validator.validate_parent(parent, has_first_rank=True)

        assert result is False

    def test_validate_sibling_valid_no_higher_ranks(self, validator, sample_decedent):
        """兄弟姉妹の検証テスト（有効・上位順位なし）"""
        validator.set_decedent(sample_decedent)

        sibling = Person(name="兄", is_alive=True, gender=Gender.MALE)

        result = validator.validate_sibling(sibling, has_first_rank=False, has_second_rank=False)

        assert result is True

    def test_validate_sibling_invalid_has_first_rank(self, validator, sample_decedent):
        """兄弟姉妹の検証テスト（無効・第1順位あり）"""
        validator.set_decedent(sample_decedent)

        sibling = Person(name="妹", is_alive=True, gender=Gender.FEMALE)

        result = validator.validate_sibling(sibling, has_first_rank=True, has_second_rank=False)

        assert result is False

    def test_validate_sibling_invalid_has_second_rank(self, validator, sample_decedent):
        """兄弟姉妹の検証テスト（無効・第2順位あり）"""
        validator.set_decedent(sample_decedent)

        sibling = Person(name="兄", is_alive=True, gender=Gender.MALE)

        result = validator.validate_sibling(sibling, has_first_rank=False, has_second_rank=True)

        assert result is False

    def test_can_substitute_without_decedent(self, validator):
        """被相続人未設定での代襲相続チェックテスト"""
        deceased_heir = Person(name="死亡相続人", is_alive=False)
        substitute = Person(name="代襲者", is_alive=True)

        with pytest.raises(ValueError) as exc_info:
            validator.can_substitute(deceased_heir, substitute, HeritageRank.FIRST)

        assert "Decedent must be set" in str(exc_info.value)

    def test_can_substitute_dead_substitute(self, validator, sample_decedent):
        """代襲相続人候補が死亡している場合のテスト"""
        validator.set_decedent(sample_decedent)

        deceased_heir = Person(
            name="死亡相続人",
            is_alive=False,
            death_date=date(2020, 1, 1)
        )
        dead_substitute = Person(
            name="死亡代襲者",
            is_alive=False,
            death_date=date(2023, 1, 1)
        )

        result = validator.can_substitute(deceased_heir, dead_substitute, HeritageRank.FIRST)

        assert result is False

    def test_can_substitute_invalid_substitution_reason(self, validator, sample_decedent):
        """代襲原因が不適切な場合のテスト"""
        validator.set_decedent(sample_decedent)

        # 被相続人死亡後に死亡（代襲原因にならない）
        deceased_heir = Person(
            name="後に死亡",
            is_alive=False,
            death_date=date(2025, 7, 1)  # 被相続人より後
        )
        substitute = Person(name="代襲者", is_alive=True)

        result = validator.can_substitute(deceased_heir, substitute, HeritageRank.FIRST)

        assert result is False

    def test_can_substitute_renounced_no_substitution(self, validator, sample_decedent):
        """相続放棄は代襲原因にならないテスト"""
        validator.set_decedent(sample_decedent)

        renounced_heir = Person(
            name="放棄した子",
            is_alive=True
        )
        validator.renounced_persons = [renounced_heir]

        substitute = Person(name="孫", is_alive=True)

        result = validator.can_substitute(renounced_heir, substitute, HeritageRank.FIRST)

        assert result is False

    def test_can_substitute_first_rank_valid(self, validator, sample_decedent):
        """第1順位の代襲相続（有効）テスト"""
        validator.set_decedent(sample_decedent)

        deceased_child = Person(
            name="先に死亡した子",
            is_alive=False,
            death_date=date(2020, 1, 1)
        )
        grandchild = Person(name="孫", is_alive=True)

        result = validator.can_substitute(deceased_child, grandchild, HeritageRank.FIRST)

        assert result is True

    def test_can_substitute_third_rank_valid(self, validator, sample_decedent):
        """第3順位の代襲相続（有効）テスト"""
        validator.set_decedent(sample_decedent)

        deceased_sibling = Person(
            name="先に死亡した兄",
            is_alive=False,
            death_date=date(2020, 1, 1)
        )
        nephew = Person(name="甥", is_alive=True)

        result = validator.can_substitute(deceased_sibling, nephew, HeritageRank.THIRD)

        assert result is True

    def test_can_substitute_disqualified_valid(self, validator, sample_decedent):
        """相続欠格者の代襲相続（有効）テスト"""
        validator.set_decedent(sample_decedent)

        disqualified_child = Person(name="欠格した子", is_alive=True)
        validator.disqualified_persons = [disqualified_child]

        grandchild = Person(name="孫", is_alive=True)

        result = validator.can_substitute(disqualified_child, grandchild, HeritageRank.FIRST)

        assert result is True

    def test_can_substitute_disinherited_valid(self, validator, sample_decedent):
        """相続廃除者の代襲相続（有効）テスト"""
        validator.set_decedent(sample_decedent)

        disinherited_child = Person(name="廃除された子", is_alive=True)
        validator.disinherited_persons = [disinherited_child]

        grandchild = Person(name="孫", is_alive=True)

        result = validator.can_substitute(disinherited_child, grandchild, HeritageRank.FIRST)

        assert result is True

    def test_can_substitute_second_rank_invalid(self, validator, sample_decedent):
        """第2順位は代襲相続できないテスト"""
        validator.set_decedent(sample_decedent)

        deceased_parent = Person(
            name="死亡した父",
            is_alive=False,
            death_date=date(2020, 1, 1)
        )
        sibling_of_decedent = Person(name="被相続人の兄弟", is_alive=True)

        result = validator.can_substitute(deceased_parent, sibling_of_decedent, HeritageRank.SECOND)

        assert result is False

    def test_get_substitution_type_first_rank(self, validator):
        """第1順位の代襲タイプ取得テスト"""
        result = validator.get_substitution_type(HeritageRank.FIRST)

        assert result == SubstitutionType.CHILD

    def test_get_substitution_type_third_rank(self, validator):
        """第3順位の代襲タイプ取得テスト"""
        result = validator.get_substitution_type(HeritageRank.THIRD)

        assert result == SubstitutionType.SIBLING

    def test_get_substitution_type_second_rank(self, validator):
        """第2順位の代襲タイプ取得テスト（代襲なし）"""
        result = validator.get_substitution_type(HeritageRank.SECOND)

        assert result == SubstitutionType.NONE

    def test_get_substitution_type_spouse(self, validator):
        """配偶者の代襲タイプ取得テスト（代襲なし）"""
        result = validator.get_substitution_type(HeritageRank.SPOUSE)

        assert result == SubstitutionType.NONE



"""相続情報収集用プロンプトテンプレート

AIエージェントが使用する質問と応答のプロンプトを定義する。
"""
from typing import Dict, List, Optional
from datetime import date


class InheritancePrompts:
    """相続情報収集用のプロンプトテンプレート集"""

    # システムプロンプト
    SYSTEM_PROMPT = """あなたは日本の相続法に精通した専門の相続アシスタントです。
被相続人（故人）の相続に関する情報を丁寧に収集し、法定相続人と相続割合を正確に計算するお手伝いをします。

あなたの役割:
1. ユーザーから相続に関する情報を段階的に収集する
2. 必要に応じて法的な用語を分かりやすく説明する
3. 民法の規定に基づいて正確な情報を提供する
4. ユーザーの回答を確認し、必要なら再質問する

重要な注意事項:
- 法的助言は提供せず、情報収集と計算のみを行う
- 複雑なケースでは弁護士や司法書士への相談を推奨する
- ユーザーの個人情報を尊重し、丁寧な言葉遣いを心がける
- 回答は簡潔かつ明確にする"""

    # 被相続人情報収集
    DECEDENT_INTRO = """これから被相続人（亡くなられた方）の相続に関する情報を収集させていただきます。
順番に質問してまいりますので、お分かりになる範囲でお答えください。

まず、被相続人の基本情報からお伺いします。"""

    DECEDENT_NAME = """被相続人（亡くなられた方）のお名前をお教えください。

（例: 山田太郎）"""

    DECEDENT_DEATH_DATE = """被相続人の死亡日（相続開始日）をお教えください。

形式: YYYY-MM-DD または YYYY年MM月DD日
（例: 2025-06-15 または 2025年6月15日）"""

    DECEDENT_BIRTH_DATE = """被相続人の生年月日をお教えください（任意）。

形式: YYYY-MM-DD または YYYY年MM月DD日
（例: 1950-01-01 または 1950年1月1日）

※分からない場合は「不明」とご入力ください。"""

    # 配偶者情報収集
    SPOUSE_QUESTION = """被相続人に配偶者（法律上の婚姻関係にある方）はいらっしゃいますか？

※内縁関係の方は法定相続人には含まれません。
※離婚された元配偶者も法定相続人には含まれません。

回答: はい / いいえ"""

    SPOUSE_INFO = """配偶者の情報をお教えください。

1. お名前:
2. 現在存命ですか？（はい/いいえ）:
3. 生年月日（分かる場合）:"""

    # 子の情報収集
    CHILDREN_QUESTION = """被相続人にお子様（実子・養子を含む）はいらっしゃいますか？

※胎児も相続人となりますが、生きて生まれることが条件です。
※養子縁組されたお子様も実子と同様に相続人となります。

回答: はい / いいえ"""

    CHILDREN_COUNT = """お子様は何人いらっしゃいますか？

人数: """

    CHILD_INFO_TEMPLATE = """【{index}人目のお子様の情報】

1. お名前:
2. 現在存命ですか？（はい/いいえ）:
3. 生年月日（分かる場合）:
4. 実子ですか、養子ですか？（実子/養子）:
5. 被相続人より先に亡くなっている場合、そのお子様（被相続人の孫）はいますか？（はい/いいえ/該当なし）:"""

    # 代襲相続の確認
    SUBSTITUTION_GRANDCHILDREN = """{child_name}さんのお子様（被相続人の孫）について教えてください。

※{child_name}さんが被相続人より先に亡くなっている場合、そのお子様が代襲相続人となります。

お子様の人数: """

    GRANDCHILD_INFO_TEMPLATE = """【{child_name}さんのお子様 {index}人目の情報】

1. お名前:
2. 現在存命ですか？（はい/いいえ）:
3. 生年月日（分かる場合）:"""

    # 直系尊属（父母・祖父母）の情報収集
    PARENTS_QUESTION = """被相続人のご両親（父母）、またはご健在の祖父母はいらっしゃいますか？

※第1順位の相続人（お子様）がいない場合、第2順位として直系尊属が相続人となります。
※父母がいらっしゃる場合、祖父母は相続人になりません。

回答: はい / いいえ"""

    PARENT_INFO_TEMPLATE = """【直系尊属の情報】

1. 続柄（父/母/祖父/祖母）:
2. お名前:
3. 現在存命ですか？（はい/いいえ）:
4. 生年月日（分かる場合）:"""

    # 兄弟姉妹の情報収集
    SIBLINGS_QUESTION = """被相続人にご兄弟姉妹はいらっしゃいますか？

※第1順位（お子様）および第2順位（直系尊属）の相続人がいない場合のみ、
　兄弟姉妹が第3順位の相続人となります。

回答: はい / いいえ"""

    SIBLINGS_COUNT = """ご兄弟姉妹は何人いらっしゃいますか？

人数: """

    SIBLING_INFO_TEMPLATE = """【{index}人目のご兄弟姉妹の情報】

1. お名前:
2. 続柄（兄/弟/姉/妹）:
3. 現在存命ですか？（はい/いいえ）:
4. 父母との関係（両親とも同じ/父のみ同じ/母のみ同じ）:
   ※これは全血・半血の判定に使用します
5. 被相続人より先に亡くなっている場合、そのお子様（甥・姪）はいますか？（はい/いいえ/該当なし）:"""

    # 相続放棄・欠格・廃除の確認
    RENUNCIATION_QUESTION = """相続放棄をされた方はいらっしゃいますか？

※相続放棄は家庭裁判所に申述して初めて効力が生じます。
※相続放棄をした方は、初めから相続人でなかったものとみなされます。

回答: はい / いいえ"""

    RENUNCIATION_INFO = """相続放棄をされた方のお名前を教えてください（複数いる場合はカンマ区切り）:

例: 山田一郎, 山田二郎"""

    DISQUALIFICATION_QUESTION = """相続欠格者（相続人となれない事由がある方）はいらっしゃいますか？

相続欠格事由の例:
- 被相続人や先順位・同順位の相続人を故意に死亡させた、またはさせようとした
- 被相続人が殺害されたことを知りながら告発・告訴しなかった
- 詐欺・強迫により被相続人の遺言を妨げた、取り消させた、変更させた

回答: はい / いいえ"""

    DISINHERITANCE_QUESTION = """相続廃除された方はいらっしゃいますか？

※相続廃除は、被相続人に対する虐待や重大な侮辱があった場合に、
　家庭裁判所の審判によって相続権を剥奪する制度です。

回答: はい / いいえ"""

    # 再転相続の確認
    RETRANSFER_QUESTION = """遺産分割協議が終わる前に亡くなられた相続人はいらっしゃいますか？

※この場合、その相続人の相続分は、その相続人の相続人に引き継がれます（再転相続）。
※これは代襲相続とは異なります。

回答: はい / いいえ"""

    RETRANSFER_INFO_TEMPLATE = """{heir_name}さんは遺産分割前に亡くなられました。
{heir_name}さんの相続人について教えてください。

1. 配偶者はいますか？（はい/いいえ）:
2. お子様はいますか？（はい/いいえ）:
3. いらっしゃる場合、それぞれのお名前:"""

    # 確認と完了
    CONFIRMATION = """以下の情報で間違いございませんか？

{summary}

確認: はい / いいえ / 修正する項目を教えてください"""

    CALCULATION_START = """情報の収集が完了しました。
これより法定相続人の確定と相続割合の計算を行います。

計算を開始してもよろしいですか？

回答: はい / いいえ"""

    COMPLETION = """相続計算が完了しました。
結果は以下の通りです。

{result}

この結果は法定相続分に基づくものです。
実際の遺産分割は、相続人全員の協議によって決定されます。

※本計算結果は参考情報であり、法的助言ではありません。
※実際の相続手続きについては、弁護士や司法書士にご相談ください。"""

    @staticmethod
    def format_child_info(index: int) -> str:
        """子の情報収集プロンプトをフォーマット"""
        return InheritancePrompts.CHILD_INFO_TEMPLATE.format(index=index)

    @staticmethod
    def format_grandchild_info(child_name: str, index: int) -> str:
        """孫の情報収集プロンプトをフォーマット"""
        return InheritancePrompts.GRANDCHILD_INFO_TEMPLATE.format(
            child_name=child_name,
            index=index
        )

    @staticmethod
    def format_sibling_info(index: int) -> str:
        """兄弟姉妹の情報収集プロンプトをフォーマット"""
        return InheritancePrompts.SIBLING_INFO_TEMPLATE.format(index=index)

    @staticmethod
    def format_confirmation(summary: str) -> str:
        """確認プロンプトをフォーマット"""
        return InheritancePrompts.CONFIRMATION.format(summary=summary)

    @staticmethod
    def format_completion(result: str) -> str:
        """完了メッセージをフォーマット"""
        return InheritancePrompts.COMPLETION.format(result=result)


# 質問フローの定義
QUESTION_FLOW = [
    "decedent_intro",
    "decedent_name",
    "decedent_death_date",
    "decedent_birth_date",
    "spouse_question",
    "children_question",
    "parents_question",
    "siblings_question",
    "renunciation_question",
    "retransfer_question",
    "confirmation",
    "calculation_start",
]



"""相続計算サービスのテスト"""
from datetime import date
from fractions import Fraction
import pytest

from inheritance_calculator_core.models.person import Person
from inheritance_calculator_core.models.relationship import BloodType
from inheritance_calculator_core.models.inheritance import HeritageRank
from inheritance_calculator_core.services.inheritance_calculator import InheritanceCalculator


class TestInheritanceCalculatorBasic:
    """基本的な相続計算のテスト"""

    def test_spouse_only(self):
        """配偶者のみの場合"""
        calculator = InheritanceCalculator()

        decedent = Person(
            name="被相続人",
            is_decedent=True,
            is_alive=False,
            death_date=date(2025, 6, 15)
        )
        spouse = Person(name="配偶者", is_alive=True)

        result = calculator.calculate(
            decedent=decedent,
            spouses=[spouse],
            children=[],
            parents=[],
            siblings=[]
        )

        assert result.total_heirs == 1
        assert result.has_spouse is True
        assert result.heirs[0].person.name == "配偶者"
        assert result.heirs[0].rank == HeritageRank.SPOUSE
        assert result.heirs[0].share == Fraction(1, 1)
        assert any("民法890条" in basis for basis in result.calculation_basis)

    def test_spouse_and_children(self):
        """配偶者と子の場合"""
        calculator = InheritanceCalculator()

        decedent = Person(name="被相続人", is_decedent=True, is_alive=False)
        spouse = Person(name="配偶者", is_alive=True)
        child1 = Person(name="子1", is_alive=True)
        child2 = Person(name="子2", is_alive=True)

        result = calculator.calculate(
            decedent=decedent,
            spouses=[spouse],
            children=[child1, child2],
            parents=[],
            siblings=[]
        )

        assert result.total_heirs == 3
        assert result.has_spouse is True
        assert result.has_children is True

        # 配偶者: 1/2
        spouse_heir = result.get_heirs_by_rank(HeritageRank.SPOUSE)[0]
        assert spouse_heir.share == Fraction(1, 2)

        # 子: 各1/4
        children_heirs = result.get_heirs_by_rank(HeritageRank.FIRST)
        assert len(children_heirs) == 2
        for heir in children_heirs:
            assert heir.share == Fraction(1, 4)

        assert any("民法900条1号" in basis for basis in result.calculation_basis)

    def test_spouse_and_parents(self):
        """配偶者と直系尊属の場合"""
        calculator = InheritanceCalculator()

        decedent = Person(name="被相続人", is_decedent=True, is_alive=False)
        spouse = Person(name="配偶者", is_alive=True)
        father = Person(name="父", is_alive=True)
        mother = Person(name="母", is_alive=True)

        result = calculator.calculate(
            decedent=decedent,
            spouses=[spouse],
            children=[],  # 子なし
            parents=[father, mother],
            siblings=[]
        )

        assert result.total_heirs == 3
        assert result.has_spouse is True
        assert result.has_parents is True
        assert result.has_children is False

        # 配偶者: 2/3
        spouse_heir = result.get_heirs_by_rank(HeritageRank.SPOUSE)[0]
        assert spouse_heir.share == Fraction(2, 3)

        # 直系尊属: 各1/6
        parent_heirs = result.get_heirs_by_rank(HeritageRank.SECOND)
        assert len(parent_heirs) == 2
        for heir in parent_heirs:
            assert heir.share == Fraction(1, 6)

        assert any("民法889条1項1号" in basis for basis in result.calculation_basis)
        assert any("民法900条2号" in basis for basis in result.calculation_basis)

    def test_spouse_and_siblings(self):
        """配偶者と兄弟姉妹の場合"""
        calculator = InheritanceCalculator()

        decedent = Person(name="被相続人", is_decedent=True, is_alive=False)
        spouse = Person(name="配偶者", is_alive=True)
        brother = Person(name="兄", is_alive=True)
        sister = Person(name="妹", is_alive=True)

        result = calculator.calculate(
            decedent=decedent,
            spouses=[spouse],
            children=[],  # 子なし
            parents=[],   # 直系尊属なし
            siblings=[brother, sister]
        )

        assert result.total_heirs == 3
        assert result.has_spouse is True
        assert result.has_siblings is True
        assert result.has_children is False
        assert result.has_parents is False

        # 配偶者: 3/4
        spouse_heir = result.get_heirs_by_rank(HeritageRank.SPOUSE)[0]
        assert spouse_heir.share == Fraction(3, 4)

        # 兄弟姉妹: 各1/8
        sibling_heirs = result.get_heirs_by_rank(HeritageRank.THIRD)
        assert len(sibling_heirs) == 2
        for heir in sibling_heirs:
            assert heir.share == Fraction(1, 8)

        assert any("民法889条1項2号" in basis for basis in result.calculation_basis)
        assert any("民法900条3号" in basis for basis in result.calculation_basis)


class TestInheritanceCalculatorRankPriority:
    """相続順位の優先順位テスト"""

    def test_first_rank_excludes_second_rank(self):
        """第1順位がいる場合、第2順位は相続できない"""
        calculator = InheritanceCalculator()

        decedent = Person(name="被相続人", is_decedent=True, is_alive=False)
        child = Person(name="子", is_alive=True)
        father = Person(name="父", is_alive=True)

        result = calculator.calculate(
            decedent=decedent,
            spouses=[],
            children=[child],
            parents=[father],
            siblings=[]
        )

        assert result.total_heirs == 1
        assert result.has_children is True
        assert result.has_parents is False  # 父は相続できない

        child_heir = result.get_heirs_by_rank(HeritageRank.FIRST)[0]
        assert child_heir.share == Fraction(1, 1)  # 全部相続

    def test_first_rank_excludes_third_rank(self):
        """第1順位がいる場合、第3順位は相続できない"""
        calculator = InheritanceCalculator()

        decedent = Person(name="被相続人", is_decedent=True, is_alive=False)
        child = Person(name="子", is_alive=True)
        brother = Person(name="兄", is_alive=True)

        result = calculator.calculate(
            decedent=decedent,
            spouses=[],
            children=[child],
            parents=[],
            siblings=[brother]
        )

        assert result.total_heirs == 1
        assert result.has_children is True
        assert result.has_siblings is False  # 兄は相続できない

    def test_second_rank_excludes_third_rank(self):
        """第2順位がいる場合、第3順位は相続できない"""
        calculator = InheritanceCalculator()

        decedent = Person(name="被相続人", is_decedent=True, is_alive=False)
        father = Person(name="父", is_alive=True)
        brother = Person(name="兄", is_alive=True)

        result = calculator.calculate(
            decedent=decedent,
            spouses=[],
            children=[],  # 子なし
            parents=[father],
            siblings=[brother]
        )

        assert result.total_heirs == 1
        assert result.has_parents is True
        assert result.has_siblings is False  # 兄は相続できない


class TestInheritanceCalculatorSiblingBloodTypes:
    """兄弟姉妹の血縁タイプテスト"""

    def test_full_blood_siblings_only(self):
        """全血兄弟姉妹のみの場合"""
        calculator = InheritanceCalculator()

        decedent = Person(name="被相続人", is_decedent=True, is_alive=False)
        brother = Person(name="兄", is_alive=True)
        sister = Person(name="妹", is_alive=True)

        blood_types = {
            brother.id: BloodType.FULL,
            sister.id: BloodType.FULL,
        }

        result = calculator.calculate(
            decedent=decedent,
            spouses=[],
            children=[],
            parents=[],
            siblings=[brother, sister],
            sibling_blood_types=blood_types
        )

        sibling_heirs = result.get_heirs_by_rank(HeritageRank.THIRD)
        assert len(sibling_heirs) == 2
        # 均等に分割
        for heir in sibling_heirs:
            assert heir.share == Fraction(1, 2)

    def test_mixed_blood_siblings(self):
        """全血と半血が混在する場合"""
        calculator = InheritanceCalculator()

        decedent = Person(name="被相続人", is_decedent=True, is_alive=False)
        full_blood_brother = Person(name="全血兄", is_alive=True)
        half_blood_brother = Person(name="半血兄", is_alive=True)

        blood_types = {
            full_blood_brother.id: BloodType.FULL,
            half_blood_brother.id: BloodType.HALF,
        }

        result = calculator.calculate(
            decedent=decedent,
            spouses=[],
            children=[],
            parents=[],
            siblings=[full_blood_brother, half_blood_brother],
            sibling_blood_types=blood_types
        )

        sibling_heirs = result.get_heirs_by_rank(HeritageRank.THIRD)
        assert len(sibling_heirs) == 2

        # 全血: 2/3, 半血: 1/3
        full_heir = [h for h in sibling_heirs if h.person.name == "全血兄"][0]
        half_heir = [h for h in sibling_heirs if h.person.name == "半血兄"][0]

        assert full_heir.share == Fraction(2, 3)
        assert half_heir.share == Fraction(1, 3)


class TestInheritanceCalculatorRenunciation:
    """相続放棄のテスト"""

    def test_child_renounced(self):
        """子が相続放棄した場合"""
        calculator = InheritanceCalculator()

        decedent = Person(name="被相続人", is_decedent=True, is_alive=False)
        child1 = Person(name="子1", is_alive=True)
        child2 = Person(name="子2", is_alive=True)
        child3_renounced = Person(name="子3（放棄）", is_alive=True)

        result = calculator.calculate(
            decedent=decedent,
            spouses=[],
            children=[child1, child2, child3_renounced],
            parents=[],
            siblings=[],
            renounced=[child3_renounced]
        )

        # 放棄していない子のみが相続
        assert result.total_heirs == 2
        heirs = result.get_heirs_by_rank(HeritageRank.FIRST)
        assert len(heirs) == 2
        assert all(h.person.name != "子3（放棄）" for h in heirs)

        # 均等に分割（1/2ずつ）
        for heir in heirs:
            assert heir.share == Fraction(1, 2)


class TestInheritanceCalculatorStringRepresentation:
    """文字列表現のテスト"""

    def test_result_string_output(self):
        """相続結果の文字列出力"""
        calculator = InheritanceCalculator()

        decedent = Person(
            name="山田太郎",
            is_decedent=True,
            is_alive=False,
            birth_date=date(1950, 1, 1),
            death_date=date(2025, 6, 15)
        )
        spouse = Person(
            name="山田花子",
            is_alive=True,
            birth_date=date(1955, 3, 10)
        )
        child = Person(
            name="山田一郎",
            is_alive=True,
            birth_date=date(1980, 5, 20)
        )

        result = calculator.calculate(
            decedent=decedent,
            spouses=[spouse],
            children=[child],
            parents=[],
            siblings=[]
        )

        output = str(result)

        # 基本情報が含まれているか
        assert "山田太郎" in output
        assert "山田花子" in output
        assert "山田一郎" in output
        assert "相続人総数: 2名" in output
        assert "1/2" in output
        assert "民法" in output



"""リポジトリパターンの実装

Neo4jデータベースへのデータアクセスを抽象化するリポジトリクラス。
"""
from typing import List, Optional, Dict, Any
from datetime import date
import logging

from .neo4j_client import Neo4jClient
from .queries import PersonQueries, RelationshipQueries, InheritanceQueries, build_person_params
from ..models.person import Person, Gender
from ..models.relationship import BloodType
from ..utils.exceptions import DatabaseException


class PersonRepository:
    """
    人物(Person)のリポジトリ

    Neo4jとPydanticモデル間の変換を行う。
    """

    def __init__(self, client: Neo4jClient) -> None:
        """
        初期化

        Args:
            client: Neo4jクライアント
        """
        self.client = client
        self.logger = logging.getLogger(__name__)

    def create(self, person: Person) -> Person:
        """
        人物を作成

        Args:
            person: 作成する人物

        Returns:
            作成された人物

        Raises:
            DatabaseException: 作成に失敗した場合
        """
        params = build_person_params(
            name=person.name,
            is_alive=person.is_alive,
            is_decedent=person.is_decedent,
            birth_date=person.birth_date.isoformat() if person.birth_date else None,
            death_date=person.death_date.isoformat() if person.death_date else None,
            gender=person.gender.value if person.gender else None,
            address=person.address,
            phone=person.phone,
            email=person.email
        )

        try:
            result = self.client.execute(PersonQueries.CREATE, params)
            self.logger.info(f"Created person: {person.name}")
            return person

        except Exception as e:
            self.logger.error(f"Failed to create person {person.name}: {e}")
            raise DatabaseException(f"人物作成エラー: {str(e)}")

    def find_by_name(self, name: str) -> Optional[Person]:
        """
        名前で人物を検索

        Args:
            name: 氏名

        Returns:
            見つかった人物、存在しない場合はNone
        """
        try:
            result = self.client.execute(PersonQueries.FIND_BY_NAME, {"name": name})

            if not result:
                return None

            return self._to_person(result[0]["p"])

        except Exception as e:
            self.logger.error(f"Failed to find person {name}: {e}")
            raise DatabaseException(f"人物検索エラー: {str(e)}")

    def find_decedent(self) -> Optional[Person]:
        """
        被相続人を検索

        Returns:
            被相続人、存在しない場合はNone
        """
        try:
            result = self.client.execute(PersonQueries.FIND_DECEDENT)

            if not result:
                return None

            return self._to_person(result[0]["p"])

        except Exception as e:
            self.logger.error(f"Failed to find decedent: {e}")
            raise DatabaseException(f"被相続人検索エラー: {str(e)}")

    def find_all(self) -> List[Person]:
        """
        全ての人物を取得

        Returns:
            人物のリスト
        """
        try:
            result = self.client.execute(PersonQueries.FIND_ALL)
            return [self._to_person(record["p"]) for record in result]

        except Exception as e:
            self.logger.error(f"Failed to find all persons: {e}")
            raise DatabaseException(f"人物一覧取得エラー: {str(e)}")

    def update(self, person: Person) -> Person:
        """
        人物を更新

        Args:
            person: 更新する人物

        Returns:
            更新された人物

        Raises:
            DatabaseException: 更新に失敗した場合
        """
        params = {
            "name": person.name,
            "is_alive": person.is_alive,
            "death_date": person.death_date.isoformat() if person.death_date else None,
            "gender": person.gender.value if person.gender else None,
            "address": person.address,
            "phone": person.phone,
            "email": person.email
        }

        try:
            result = self.client.execute(PersonQueries.UPDATE, params)

            if not result:
                raise DatabaseException(f"人物が見つかりません: {person.name}")

            self.logger.info(f"Updated person: {person.name}")
            return person

        except Exception as e:
            self.logger.error(f"Failed to update person {person.name}: {e}")
            raise DatabaseException(f"人物更新エラー: {str(e)}")

    def delete(self, name: str) -> None:
        """
        人物を削除

        Args:
            name: 削除する人物の氏名

        Raises:
            DatabaseException: 削除に失敗した場合
        """
        try:
            self.client.execute(PersonQueries.DELETE, {"name": name})
            self.logger.info(f"Deleted person: {name}")

        except Exception as e:
            self.logger.error(f"Failed to delete person {name}: {e}")
            raise DatabaseException(f"人物削除エラー: {str(e)}")

    def delete_all(self) -> None:
        """
        全ての人物を削除（テスト用）

        Warning:
            本番環境では使用しないこと
        """
        try:
            self.client.execute(PersonQueries.DELETE_ALL)
            self.logger.warning("Deleted all persons")

        except Exception as e:
            self.logger.error(f"Failed to delete all persons: {e}")
            raise DatabaseException(f"全人物削除エラー: {str(e)}")

    def _to_person(self, node: Dict[str, Any]) -> Person:
        """
        Neo4jノードをPersonモデルに変換

        Args:
            node: Neo4jノード

        Returns:
            Personモデル
        """
        # Neo4jのdateオブジェクトをPythonのdateに変換
        birth_date = None
        if node.get("birth_date"):
            bd = node["birth_date"]
            birth_date = date(bd.year, bd.month, bd.day) if hasattr(bd, "year") else None

        death_date = None
        if node.get("death_date"):
            dd = node["death_date"]
            death_date = date(dd.year, dd.month, dd.day) if hasattr(dd, "year") else None

        gender = Gender(node["gender"]) if node.get("gender") else Gender.UNKNOWN

        return Person(
            name=node["name"],
            is_alive=node["is_alive"],
            is_decedent=node["is_decedent"],
            birth_date=birth_date,
            death_date=death_date,
            gender=gender,
            address=node.get("address"),
            phone=node.get("phone"),
            email=node.get("email")
        )


class RelationshipRepository:
    """
    リレーションシップのリポジトリ

    親子関係、配偶者関係、兄弟姉妹関係などを管理する。
    """

    def __init__(self, client: Neo4jClient) -> None:
        """
        初期化

        Args:
            client: Neo4jクライアント
        """
        self.client = client
        self.person_repo = PersonRepository(client)
        self.logger = logging.getLogger(__name__)

    def create_child_of(
        self,
        child_name: str,
        parent_name: str,
        adoption: bool = False,
        is_biological: bool = True
    ) -> None:
        """
        親子関係を作成

        Args:
            child_name: 子の氏名
            parent_name: 親の氏名
            adoption: 養子縁組フラグ
            is_biological: 実子フラグ
        """
        params = {
            "child_name": child_name,
            "parent_name": parent_name,
            "adoption": adoption,
            "is_biological": is_biological
        }

        try:
            self.client.execute(RelationshipQueries.CREATE_CHILD_OF, params)
            self.logger.info(f"Created CHILD_OF: {child_name} -> {parent_name}")

        except Exception as e:
            self.logger.error(f"Failed to create CHILD_OF relationship: {e}")
            raise DatabaseException(f"親子関係作成エラー: {str(e)}")

    def create_spouse_of(
        self,
        person1_name: str,
        person2_name: str,
        marriage_date: Optional[date] = None,
        divorce_date: Optional[date] = None,
        is_current: bool = True
    ) -> None:
        """
        配偶者関係を作成

        Args:
            person1_name: 配偶者1の氏名
            person2_name: 配偶者2の氏名
            marriage_date: 婚姻日
            divorce_date: 離婚日
            is_current: 現在の配偶者フラグ
        """
        params = {
            "person1_name": person1_name,
            "person2_name": person2_name,
            "marriage_date": marriage_date.isoformat() if marriage_date else None,
            "divorce_date": divorce_date.isoformat() if divorce_date else None,
            "is_current": is_current
        }

        try:
            self.client.execute(RelationshipQueries.CREATE_SPOUSE_OF, params)
            self.logger.info(f"Created SPOUSE_OF: {person1_name} <-> {person2_name}")

        except Exception as e:
            self.logger.error(f"Failed to create SPOUSE_OF relationship: {e}")
            raise DatabaseException(f"配偶者関係作成エラー: {str(e)}")

    def create_sibling_of(
        self,
        person1_name: str,
        person2_name: str,
        blood_type: BloodType = BloodType.FULL,
        shared_parent: str = "both"
    ) -> None:
        """
        兄弟姉妹関係を作成

        Args:
            person1_name: 兄弟姉妹1の氏名
            person2_name: 兄弟姉妹2の氏名
            blood_type: 血縁タイプ
            shared_parent: 共通の親
        """
        params = {
            "person1_name": person1_name,
            "person2_name": person2_name,
            "blood_type": blood_type.value,
            "shared_parent": shared_parent
        }

        try:
            self.client.execute(RelationshipQueries.CREATE_SIBLING_OF, params)
            self.logger.info(f"Created SIBLING_OF: {person1_name} <-> {person2_name}")

        except Exception as e:
            self.logger.error(f"Failed to create SIBLING_OF relationship: {e}")
            raise DatabaseException(f"兄弟姉妹関係作成エラー: {str(e)}")

    def create_renounced(
        self,
        person_name: str,
        decedent_name: str,
        renounce_date: date,
        reason: Optional[str] = None
    ) -> None:
        """
        相続放棄関係を作成

        Args:
            person_name: 放棄者の氏名
            decedent_name: 被相続人の氏名
            renounce_date: 放棄日
            reason: 理由
        """
        params = {
            "person_name": person_name,
            "decedent_name": decedent_name,
            "renounce_date": renounce_date.isoformat(),
            "reason": reason
        }

        try:
            self.client.execute(RelationshipQueries.CREATE_RENOUNCED, params)
            self.logger.info(f"Created RENOUNCED: {person_name} -> {decedent_name}")

        except Exception as e:
            self.logger.error(f"Failed to create RENOUNCED relationship: {e}")
            raise DatabaseException(f"相続放棄関係作成エラー: {str(e)}")

    def find_children(self, parent_name: str) -> List[Person]:
        """
        親の子を検索

        Args:
            parent_name: 親の氏名

        Returns:
            子のリスト
        """
        try:
            result = self.client.execute(
                RelationshipQueries.FIND_CHILDREN,
                {"parent_name": parent_name}
            )
            return [self.person_repo._to_person(record["child"]) for record in result]

        except Exception as e:
            self.logger.error(f"Failed to find children of {parent_name}: {e}")
            raise DatabaseException(f"子の検索エラー: {str(e)}")

    def find_parents(self, child_name: str) -> List[Person]:
        """
        子の親を検索

        Args:
            child_name: 子の氏名

        Returns:
            親のリスト
        """
        try:
            result = self.client.execute(
                RelationshipQueries.FIND_PARENTS,
                {"child_name": child_name}
            )
            return [self.person_repo._to_person(record["parent"]) for record in result]

        except Exception as e:
            self.logger.error(f"Failed to find parents of {child_name}: {e}")
            raise DatabaseException(f"親の検索エラー: {str(e)}")

    def find_spouse(self, person_name: str) -> Optional[Person]:
        """
        配偶者を検索

        Args:
            person_name: 氏名

        Returns:
            配偶者、存在しない場合はNone
        """
        try:
            result = self.client.execute(
                RelationshipQueries.FIND_SPOUSE,
                {"person_name": person_name}
            )

            if not result:
                return None

            return self.person_repo._to_person(result[0]["spouse"])

        except Exception as e:
            self.logger.error(f"Failed to find spouse of {person_name}: {e}")
            raise DatabaseException(f"配偶者の検索エラー: {str(e)}")

    def find_siblings(self, person_name: str) -> List[tuple[Person, BloodType]]:
        """
        兄弟姉妹を検索

        Args:
            person_name: 氏名

        Returns:
            (兄弟姉妹, 血縁タイプ)のリスト
        """
        try:
            result = self.client.execute(
                RelationshipQueries.FIND_SIBLINGS,
                {"person_name": person_name}
            )

            siblings = []
            for record in result:
                sibling = self.person_repo._to_person(record["sibling"])
                blood_type = BloodType(record["r"]["blood_type"]) if "r" in record else BloodType.FULL
                siblings.append((sibling, blood_type))

            return siblings

        except Exception as e:
            self.logger.error(f"Failed to find siblings of {person_name}: {e}")
            raise DatabaseException(f"兄弟姉妹の検索エラー: {str(e)}")


class InheritanceRepository:
    """
    相続計算用のリポジトリ

    相続人の取得など、相続計算に特化したクエリを実行する。
    """

    def __init__(self, client: Neo4jClient) -> None:
        """
        初期化

        Args:
            client: Neo4jクライアント
        """
        self.client = client
        self.person_repo = PersonRepository(client)
        self.logger = logging.getLogger(__name__)

    def get_spouse(self) -> Optional[Person]:
        """
        配偶者を取得

        Returns:
            配偶者、存在しない場合はNone
        """
        try:
            result = self.client.execute(InheritanceQueries.GET_SPOUSE)

            if not result:
                return None

            return self.person_repo._to_person(result[0]["spouse"])

        except Exception as e:
            self.logger.error(f"Failed to get spouse: {e}")
            raise DatabaseException(f"配偶者取得エラー: {str(e)}")

    def get_first_rank_heirs(self) -> List[Person]:
        """
        第1順位相続人（子）を取得

        Returns:
            子のリスト
        """
        try:
            result = self.client.execute(InheritanceQueries.GET_FIRST_RANK_HEIRS)
            return [self.person_repo._to_person(record["child"]) for record in result]

        except Exception as e:
            self.logger.error(f"Failed to get first rank heirs: {e}")
            raise DatabaseException(f"第1順位相続人取得エラー: {str(e)}")

    def get_second_rank_heirs(self) -> List[Person]:
        """
        第2順位相続人（直系尊属）を取得

        Returns:
            直系尊属のリスト
        """
        try:
            result = self.client.execute(InheritanceQueries.GET_SECOND_RANK_HEIRS)
            return [self.person_repo._to_person(record["ancestor"]) for record in result]

        except Exception as e:
            self.logger.error(f"Failed to get second rank heirs: {e}")
            raise DatabaseException(f"第2順位相続人取得エラー: {str(e)}")

    def get_third_rank_heirs(self) -> List[tuple[Person, BloodType]]:
        """
        第3順位相続人（兄弟姉妹）を取得

        Returns:
            (兄弟姉妹, 血縁タイプ)のリスト
        """
        try:
            result = self.client.execute(InheritanceQueries.GET_THIRD_RANK_HEIRS)

            heirs = []
            for record in result:
                sibling = self.person_repo._to_person(record["sibling"])
                blood_type = BloodType(record["blood_type"])
                heirs.append((sibling, blood_type))

            return heirs

        except Exception as e:
            self.logger.error(f"Failed to get third rank heirs: {e}")
            raise DatabaseException(f"第3順位相続人取得エラー: {str(e)}")
